<?php
namespace PME\Controllers\Servico;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class ConsPorServicoReport extends \Zeedhi\Framework\Controller\Report {

    protected $birtStrategy;
    protected $crudService;
    protected $lista;
    protected $utilities;

    public function __construct(ReportService $reportService, $birtStrategy, $crudService, $lista, $utilities) {
      $this->birtStrategy  = $birtStrategy;
      $this->lista         = $lista;
      $this->crudService   = $crudService;
      $this->utilities     = $utilities;

      parent::__construct($reportService);
    }

    protected $reportMapping = array (
      'PME00700' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
              "UNIDADEREPORT" => "P_CDFILIAL",
              "CONSUMO"       => "P_CONSUMO",
              "CDFILCENTPME"  => "P_CDFILCENTPME",
              "SERVICOREPORT" => "P_CDSERVFILI",
              "DTVALIDADE"    => "P_DTINIVALID",
              "DIASCONS"      => "P_IDTIPOCONS",
              "PRODUCTMASK"   => "P_PRODUCTMASK",
              "NRORG"         => "P_NRORG",
            )
        ),
        'PME00701' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
              "UNIDADEREPORT" => "P_CDFILIAL",
              "CONSUMO"       => "P_CONSUMO",
              "CDFILCENTPME"  => "P_CDFILCENTPME",
              "SERVICOREPORT" => "P_CDSERVFILI",
              "DTDE"          => "P_DTINI",
              "DTATE"         => "P_DTFIN",
              "IDTCONSSERV"   => "P_IDTCONSSERV",
              "PRODUCTMASK"   => "P_PRODUCTMASK",
              "NRORG"         => "P_NRORG",
            )
        ),
        'PME00703' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array ( 
              "CDFILCENTPME"        => "P_CDFILCENTPME",
              "CONSUMO"             => "P_CONSUMO",
              "DIASCONS"            => "P_IDTIPOCONS",
              "NRLISTASERVICO"      => "P_NRLISTASERVICO",
              "DTREFERENCIAREPORT"  => "P_DTINIVALID",
              "PRODUCTMASK"         => "P_PRODUCTMASK",
              "NRORG"               => "P_NRORG",
            )
        )
    );
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
      $row = $request->getRow();
      $nrorg = $this->utilities->getNrOrg();
      $row['CONSUMO'] = implode('_', $row['TPCONSUMO']);
      if($row['CD_REPORT_CODE'] == 'PME00700') $row['DIASCONS'] = $this->filterDaysCons($row['DIASCONSPADR']); 
      else if($row['CD_REPORT_CODE'] == 'PME00703'){
          $row['DIASCONS'] = $this->filterDaysCons($row['DIASCONSOLIDADO']);
          if(empty($row['MULTSERVICOREPORT'])) $row['MULTSERVICOREPORT'] = $this->getServicos($row['CDFILCENTPME']);
          $filiServ = '';

          foreach($row['MULTSERVICOREPORT'] as $serv)  {
            $filiServ .= $serv['CDFILIAL'].$serv['CDSERVFILI'].'_'; 
          }
          
          $row['NRLISTASERVICO'] = $this->lista->geraListaServico($filiServ, $nrorg);
      } else $row['DIASCONS'] = implode('_', $row['DIASCONSPADR']); 
      
      if($row['CD_REPORT_CODE'] == 'PME00701'){
        if($row['CONSUMO'] == 'F_P') $row['IDTCONSSERV'] = 'F_P_A';
        else $row['IDTCONSSERV'] = $row['CONSUMO'];
      } 
      $row['NRORG'] = $nrorg;
      
      $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
      $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
    }
    
    private function filterDaysCons($days) {
      $result;
      if(in_array('T', $days)) $result = '1_2_3_4_5_6_7_8';
      else if(in_array('U', $days)) {
        $key = array_search('U', $days);
        unset($days[$key]);
        $result = '2_3_4_5_6_'.implode('_', $days);
      } else $result = implode('_', $days);
      
      return $result;
    }
    
    private function getServicos($central){
      $conditions = [
        array(
          "columnName" => 'P_CDFILIAL',
          "operator"   => '=',
          "value"      => $central
        ),
        array(
          "columnName" => 'NRORG',
          "operator"   => '=',
          "value"      => $this->utilities->getNrOrg()
        )
      ];

      $dataSet = $this->crudService->getFilteredDataSet("servico", $conditions);
      return $dataSet->getRows();
    }
}