<?php
namespace PME\Modules\Planejamento\Prato;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager\Doctrine\NameProvider;

class ModoPreparoController extends \PME\Services\TeknisaCrud {
    protected $crudService;
    protected $parameterBag;
	protected $connection;
	protected $entityManager;
	protected $utilities;
	protected $environment;

    public function __construct($crudService, $parameterBag, $connection, $entityManager, $utilities, $environment) {
        parent::__construct($crudService, $parameterBag);
        $this->connection    = $connection;
		$this->entityManager = $entityManager;
		$this->utilities     = $utilities;
		$this->environment   = $environment;
	}
	
	public function saveModoPreparo(DTO\Request\Row $request, DTO\Response $response){
	    $parameters = $request->getParameters();
		$rows = $parameters['editedRows'];
        
        if (empty($rows['CDPRATO'])) {
            foreach ($rows as $row) {
                $this->modoPreparo($row, $request, $response);
            }
        } else $this->modoPreparo($rows, $request, $response);
	    
	}
	
	public function modoPreparo($row, $request, $response) {
	    try {
            $prato = $row['CDPRATO'];
    		$filAux = $row['CDFILAUXPRAT'];
    
            $nrorg = $this->utilities->getNrOrg();
            $operador = $this->environment->getCdOperador(); 
    
    		if ($row['TIPO'] == 'P') {
    		    $modoExistentes = $this->retornaPreparoPadrao($prato, $filAux);
    		    if ($modoExistentes) {
    		        $codigo = $modoExistentes[0]['NRPREPPRAT'] + 1;
    		        $trat = $this->trataCodigoPreparo($codigo);
    		        
    		        if (!$trat) {
                        $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                        return false;
                    }
                    $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
                    if ($row['IDPREPPRAT'] == '1') $this->atualizaStatusPreparoPadrao($filAux, $prato);
    		    } else $codigo = '01';

    		    if (!$row['NRPREPPRAT']) $this->inserePreparoPadrao($row, $codigo, $nrorg);
                else $this->atualizaPreparoPadrao($row, $nrorg);
    		}
    		else if ($row['TIPO'] == 'L') {
    		    $filial = $row['CDFILIAL'];
    		    $modoExistentes = $this->retornaPreparoLocal($prato, $filAux, $filial);
    		    
    		    if ($modoExistentes) {
    		        $codigo = $modoExistentes[0]['NRPREPFILI'] + 1;
    		        $trat = $this->trataCodigoPreparo($codigo);
    		        if (!$trat) {
                        $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                        return false;
                    }
                    $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
                    
                    if ($row['IDPREPFILI'] == '1') $this->atualizaStatusPreparoLocal($filAux, $prato, $filial);
    		    } else $codigo = '01';
    		    
    		    if (!$row['NRPREPFILI']) $this->inserePreparoLocal($row, $codigo, $filial, $nrorg);
    		    else $this->atualizaPreparoLocal($row, $filial, $nrorg);
    		} else if ($row['TIPO'] == 'S') {
    		    $filial = $row['CDFILIAL'];
                $servico = $row['CDSERVFILI'];
                $modoExistentes = $this->retornaPreparoServico($prato, $filAux, $filial, $servico);
                
                if ($modoExistentes) {
                    $codigo = $modoExistentes[0]['NRPREPSERV'] + 1;
                    $trat = $this->trataCodigoPreparo($codigo);
                    if (!$trat) {
                        $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                        return false;
                    }
                    $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                    if ($row['IDPREPSERV'] == '1') $this->atualizaStatusPreparoServico($filAux, $prato, $filial, $servico);
                } else $codigo = '01';
                
                if (!$row["NRPREPSERV"]) $this->inserePreparoServico($row, $filial, $servico, $codigo, $nrorg);
                else $this->atualizaPreparoServico($row, $filial, $servico, $nrorg);
    		} else if ($row['TIPO'] == 'C') {
    		    $cardapio = $row['CDTIPOCARD'];
    		    $modoExistentes = $this->retornaPreparoTipoCardapio($prato, $filAux, $cardapio);
    		    
    		    if ($modoExistentes) {
                    $codigo = $modoExistentes[0]['NRPREPTPCARD'] + 1;
                    $trat = $this->trataCodigoPreparo($codigo);
                    if (!$trat) {
                        $response->addDataset(new DataSet_Source('CALLBACK', array('ERRO_CODIGO')));
                        return false;
                    }
                    $codigo = str_pad($codigo, 2, "0", STR_PAD_LEFT);
    
                    if ($row['IDPREPTPCARD'] == '1') $this->atualizaStatusPreparoTipoCardapio($filAux, $prato, $cardapio);
                } else $codigo = '01';
                
                if (!$row["NRPREPTPCARD"]) $this->inserePreparoTipoCardapio($row, $cardapio, $codigo, $nrorg);
                else $this->atualizaPreparoTipoCardapio($row, $cardapio, $nrorg);
    		}
    		
    		$response->addDataSet(new DataSet("result", array(true)));
	    } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
	}
	
	private function trataCodigoPreparo($codigo){
        $tam = strlen($codigo);
        if ($tam > 2) return false;
        else  return true;
    }
	
	public function retornaPreparoPadrao($prato, $filAux, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO, NRPREPPRAT, IDPREPPRAT, TXPREPPRAT, NMRPREPPRAT, NRORG FROM PREPPRAPAD WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT ". $query." ORDER BY NRPREPPRAT DESC";

        $params = array(
            "CDPRATO"      => $prato,
            "CDFILAUXPRAT" => $filAux
        );
        
        $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $modos;
    }
    
    public function atualizaStatusPreparoPadrao($filAux, $prato) {
        $update = "UPDATE PREPPRAPAD SET IDPREPPRAT = :IDPREPPRAT WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO";

        $params = array(
            "CDFILAUXPRAT"=> $filAux,
            "CDPRATO"     => $prato,
            "IDPREPPRAT"  => '2'
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->execute($params);
    }
    
    public function inserePreparoPadrao($row, $codigo, $nrorg){
        $insert = "INSERT INTO PREPPRAPAD (CDFILAUXPRAT, CDPRATO, NRPREPPRAT, IDPREPPRAT, TXPREPPRAT, NMRPREPPRAT, NRORG) VALUES (:CDFILAUXPRAT, :CDPRATO, :NRPREPPRAT, :IDPREPPRAT, :TXPREPPRAT, :NMRPREPPRAT, :NRORG)";

        $params = array(
            "CDFILAUXPRAT"   => $row['CDFILAUXPRAT'],
            "CDPRATO"        => $row['CDPRATO'],
            "NRPREPPRAT"     => $codigo,
            "IDPREPPRAT"     => $row['IDPREPPRAT'],
            "NMRPREPPRAT"    => $row['NMRPREPPRAT'],
            "NRORG"          => $nrorg
        );

        $stmt = $this->entityManager->getConnection()->prepare($insert);
        $stmt->bindParam('TXPREPPRAT', $row['TXPREPPRAT'], SQLT_LBI);
        $stmt->execute($params);
    }
    
    public function atualizaPreparoPadrao($row, $nrorg) {
        $update = "UPDATE PREPPRAPAD SET IDPREPPRAT = :IDPREPPRAT, NMRPREPPRAT = :NMRPREPPRAT, TXPREPPRAT = :TXPREPPRAT WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND NRPREPPRAT = :NRPREPPRAT";

        $params = array(
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO"      => $row['CDPRATO'],
            "NRPREPPRAT"   => $row['NRPREPPRAT'],
            "IDPREPPRAT"   => 1,
            "NMRPREPPRAT"  => $row['NMRPREPPRAT'],
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->bindParam('TXPREPPRAT', $row['TXPREPPRAT'], SQLT_LBI);
        $stmt->execute($params);
    }
    
    public function retornaPreparoLocal($prato, $filAux, $filial, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, NRPREPFILI, IDPREPFILI, TXPREPFILI, NMRPREPFILI, NRORG FROM PREPPRAFIL WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDFILIAL = :CDFILIAL ". $query." ORDER BY NRPREPFILI DESC";

        $params = array(
            "CDPRATO"      => $prato,
            "CDFILAUXPRAT" => $filAux,
            "CDFILIAL"     => $filial
        );

        $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $modos;
    }
    
    public function inserePreparoLocal($row, $codigo, $filial, $nrorg) {
        $insert = "INSERT INTO PREPPRAFIL ( CDFILAUXPRAT, CDPRATO, CDFILIAL, NRPREPFILI, IDPREPFILI, TXPREPFILI, NMRPREPFILI, NRORG) VALUES ( :CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :NRPREPFILI, :IDPREPFILI, :TXPREPFILI, :NMRPREPFILI, :NRORG)";

        $params = array(
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO"      => $row['CDPRATO'],
            "CDFILIAL"     => $filial,
            "NRPREPFILI"   => $codigo,
            "IDPREPFILI"   => $row['IDPREPFILI'],
            "NMRPREPFILI"  => $row['NMRPREPFILI'],
            "NRORG"        => $nrorg
        );

        $stmt = $this->entityManager->getConnection()->prepare($insert);
        $stmt->bindParam('TXPREPFILI', $row['TXPREPFILI'], SQLT_LBI);
        $stmt->execute($params);
    }

    public function atualizaPreparoLocal($row, $filial, $nrorg) {
        $update = "UPDATE PREPPRAFIL SET IDPREPFILI = :IDPREPFILI, NMRPREPFILI = :NMRPREPFILI, TXPREPFILI = :TXPREPFILI WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDFILIAL = :CDFILIAL AND NRPREPFILI = :NRPREPFILI";

        $params = array(
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO"      => $row['CDPRATO'],
            "CDFILIAL"     => $filial,
            "NRPREPFILI"   => $row['NRPREPFILI'],
            "IDPREPFILI"   => $row['IDPREPFILI'],
            "NMRPREPFILI"  => $row['NMRPREPFILI']
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->bindParam('TXPREPFILI', $row['TXPREPFILI'], SQLT_LBI);
        $stmt->execute($params);
    }
    
    public function atualizaStatusPreparoLocal($filAux, $prato, $filial) {
        try {
            $update = "UPDATE PREPPRAFIL SET IDPREPFILI = :IDPREPFILI WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDFILIAL = :CDFILIAL";

            $params = array(
                "CDFILAUXPRAT" => $filAux,
                "CDPRATO"      => $prato,
                "CDFILIAL"     => $filial,
                "IDPREPFILI"   => '2'
            );

            $stmt = $this->entityManager->getConnection()->prepare($update);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function retornaPreparoServico($prato, $filAux, $filial, $servico, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO, CDFILIAL, CDSERVFILI, NRPREPSERV, IDPREPSERV, TXPREPSERV, NMRPREPSERV, NRORG FROM PREPPRASER WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI ".$query." ORDER BY NRPREPSERV DESC";

        $params = array(
            "CDPRATO"      => $prato,
            "CDFILAUXPRAT" => $filAux,
            "CDFILIAL"     => $filial,
            "CDSERVFILI"   => $servico
        );

        try {
            $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
            // if ($modos) {
            //     foreach ($modos as $i => $modo) {
            //         $preparo = str_replace('\pard', '<br>', $modos[$i]['TXPREPSERV']);
            //         $preparo = str_replace('\par', '<br>', $preparo);
            //         $modos[$i]['TXPREPPRAT'] = DataSourceUtil::toPlainText($preparo);
            //     }
            // }

            return $modos;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function inserePreparoServico($row, $filial, $servico, $codigo, $nrorg) {
        try {
            $insert = "INSERT INTO PREPPRASER (CDFILAUXPRAT, CDPRATO, CDFILIAL, CDSERVFILI, NRPREPSERV, IDPREPSERV, TXPREPSERV, NMRPREPSERV, NRORG) VALUES ( :CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :CDSERVFILI, :NRPREPSERV, :IDPREPSERV, :TXPREPSERV, :NMRPREPSERV, :NRORG)";

            $params = array(
                "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
                "CDPRATO"      => $row['CDPRATO'],
                "CDFILIAL"     => $filial,
                "CDSERVFILI"   => $servico,
                "NRPREPSERV"   => $codigo,
                "IDPREPSERV"   => $row['IDPREPSERV'],
                "NMRPREPSERV"  => $row['NMRPREPSERV'],
                "NRORG"        => $nrorg
            );

            $stmt = $this->entityManager->getConnection()->prepare($insert);
            $stmt->bindParam('TXPREPSERV', $row['TXPREPSERV'], SQLT_LBI);
            $stmt->execute($params);
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function atualizaPreparoServico($row, $filial, $servico, $nrorg) {
        $update = "UPDATE PREPPRASER SET IDPREPSERV = :IDPREPSERV, NMRPREPSERV = :NMRPREPSERV, TXPREPSERV = :TXPREPSERV WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND NRPREPSERV = :NRPREPSERV";

        $params = array(
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO"      => $row['CDPRATO'],
            "CDFILIAL"     => $filial,
            "CDSERVFILI"   => $servico,
            "NRPREPSERV"   => $row['NRPREPSERV'],
            "IDPREPSERV"   => $row['IDPREPSERV'],
            "NMRPREPSERV"  => $row['NMRPREPSERV'],
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->bindParam('TXPREPSERV', $row['TXPREPSERV'], SQLT_LBI);
        $stmt->execute($params);
    }
    
    public function atualizaStatusPreparoServico($filAux, $prato, $filial, $servico) {
        $update = "UPDATE PREPPRASER SET IDPREPSERV = :IDPREPSERV WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI";

        $params = array(
            "CDFILAUXPRAT" => $filAux,
            "CDPRATO"      => $prato,
            "CDFILIAL"     => $filial,
            "CDSERVFILI"   => $servico,
            "IDPREPSERV"   => '2'
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->execute($params);
    }
    
    public function retornaPreparoTipoCardapio($prato, $filAux, $cardapio, $query = null){
        $sql = "SELECT CDFILAUXPRAT, CDPRATO, CDTIPOCARD, NRPREPTPCARD, IDPREPTPCARD, TXPREPTPCARD, NMPREPTPCARD, NRORG FROM PREPPRATPCARD WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDTIPOCARD = :CDTIPOCARD ".$query." ORDER BY NRPREPTPCARD DESC";

        $params = array(
            "CDPRATO"      => $prato,
            "CDFILAUXPRAT" => $filAux,
            "CDTIPOCARD"   => $cardapio
        );

        $modos = $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $modos;
    }
    
    public function inserePreparoTipoCardapio($row, $cardapio, $codigo, $nrorg) {
        $insert = "INSERT INTO PREPPRATPCARD (CDFILCENTPME, CDFILAUXPRAT, CDPRATO, CDTIPOCARD, NRPREPTPCARD, IDPREPTPCARD, TXPREPTPCARD, NMPREPTPCARD, NRORG) VALUES (:CDFILCENTPME, :CDFILAUXPRAT, :CDPRATO, :CDTIPOCARD, :NRPREPTPCARD, :IDPREPTPCARD, :TXPREPTPCARD, :NMPREPTPCARD, :NRORG)";
        $params = array(
            "CDFILCENTPME" => $row['CDFILCENTPME'],
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO"      => $row['CDPRATO'],
            "CDTIPOCARD"   => $cardapio,
            "NRPREPTPCARD" => $codigo,
            "IDPREPTPCARD" => $row['IDPREPTPCARD'],
            "NMPREPTPCARD" => $row['NMPREPTPCARD'],
            "TXPREPTPCARD" => $row['TXPREPTPCARD'],
            "NRORG"        => $nrorg
        );

        $stmt = $this->entityManager->getConnection()->prepare($insert);
        //$stmt->bindParam('TXPREPTPCARD', $row['TXPREPTPCARD'], SQLT_LBI);
        $stmt->execute($params);
    }

    public function atualizaPreparoTipoCardapio($row, $cardapio, $nrorg) {
        $update = "UPDATE PREPPRATPCARD SET IDPREPTPCARD = :IDPREPTPCARD, NMPREPTPCARD = :NMPREPTPCARD, TXPREPTPCARD = :TXPREPTPCARD WHERE CDFILCENTPME = :CDFILCENTPME AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDTIPOCARD = :CDTIPOCARD AND NRPREPTPCARD = :NRPREPTPCARD";

        $params = array(
            "CDFILCENTPME" => $row['CDFILCENTPME'],
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO"      => $row['CDPRATO'],
            "CDTIPOCARD"   => $cardapio,
            "NRPREPTPCARD" => $row['NRPREPTPCARD'],
            "IDPREPTPCARD" => $row['IDPREPTPCARD'],
            "NMPREPTPCARD" => $row['NMPREPTPCARD'],
            "TXPREPTPCARD" => $row['TXPREPTPCARD'],
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        // $stmt->bindParam('TXPREPTPCARD', $row['TXPREPTPCARD'], SQLT_LBI);
        $stmt->execute($params);
    }
    
    public function atualizaStatusPreparoTipoCardapio($filAux, $prato, $cardapio) {
        $update = "UPDATE PREPPRATPCARD SET IDPREPTPCARD = :IDPREPTPCARD WHERE CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRATO = :CDPRATO AND CDTIPOCARD = :CDTIPOCARD";

        $params = array(
            "CDFILAUXPRAT" => $filAux,
            "CDPRATO"      => $prato,
            "CDTIPOCARD"   => $cardapio,
            "IDPREPTPCARD" => '2'
        );

        $stmt = $this->entityManager->getConnection()->prepare($update);
        $stmt->execute($params);
    }
    
    public function retornaModoPreparo(DTO\Request\Row $request, DTO\Response $response) {
        $parameters = $request->getParameters();
		$rows = $parameters['datasource'];
		$text = [];
        
        foreach ($rows as $row) {
            if (empty($row['TXPREPPRAT']) == false) $field = $row['TXPREPPRAT'];
            else if (empty($row['TXPREPTPCARD']) == false) $field = $row['TXPREPTPCARD'];
            else if (empty($row['TXPREPFILI']) == false) $field = $row['TXPREPFILI'];
            else if (empty($row['TXPREPSERV']) == false) $field = $row['TXPREPSERV'];
            
            $plain = '';
            $plain = preg_replace('"{\*?\\\\.+(;})|\\s?\\\[A-Za-z0-9]+|\\s?{\\s?\\\[A-Za-z0-9‹]+\\s?|\\s?}\\s?"', '', $field);

            $plain = str_replace("\'e1", 'á', $plain);
            $plain = str_replace("\'e3", 'ã', $plain);
            $plain = str_replace("\'ed", 'í', $plain);
            $plain = str_replace("\'fa", 'ú', $plain);
            $plain = str_replace("\'f3", 'ó', $plain);
            $plain = str_replace("\'da", 'Ú', $plain);
            $plain = str_replace("\'c3", 'Ã', $plain);
            $plain = str_replace("\'d3", 'Ó', $plain);
            $plain = str_replace("\'ca", 'Ê', $plain);
            $plain = str_replace("\'cd", 'Í', $plain);
            $plain = str_replace("\'c1", 'Á', $plain);
            $plain = str_replace("\'c7", 'Ç', $plain);
            $plain = str_replace("\'c0", 'À', $plain);
            $plain = str_replace("\'3f", '?', $plain);
            $plain = str_replace("\'c4", 'Ä', $plain);
            $plain = str_replace("\'c9", 'É', $plain);
            $plain = str_replace("\'d6", 'Ö', $plain);
            $plain = str_replace("\'dc", 'Ü', $plain);
            $plain = str_replace("\'e0", 'à', $plain);
            $plain = str_replace("\'e2", 'â', $plain);
            $plain = str_replace("\'e4", 'ä', $plain);
            $plain = str_replace("\'e7", 'ç', $plain);
            $plain = str_replace("\'e8", 'è', $plain);
            $plain = str_replace("\'e9", 'é', $plain);
            $plain = str_replace("\'ea", 'ê', $plain);
            $plain = str_replace("\'eb", 'ë', $plain);
            $plain = str_replace("\'ee", 'î', $plain);
            $plain = str_replace("\'f4", 'ô', $plain);
            $plain = str_replace("\'f6", 'ö', $plain);
            $plain = str_replace("\'fb", 'û', $plain);
            $plain = str_replace("\'fc", 'ü', $plain);
            
            $plain = trim($plain);
            $plain = preg_replace('/^-0 /', '', $plain);
            $text[] = $plain;
        }
        
        $response->addDataSet(new DataSet("result", $text));
    }
}