<?php
namespace PME\Modules\Planejamento\Prato;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class ReceitaController extends \PME\Services\TeknisaCrud {

	protected $crudService;
	protected $parameterBag;
	protected $entityManager;
	protected $connection;
	protected $environment;
  protected $newCode;
  
  public function __construct($crudService, $parameterBag, $entityManager, $connection, $environment, $newCode) {
    $this->entityManager = $entityManager;
    $this->connection = $connection;
    $this->environment = $environment;
    $this->newCode = $newCode;
    parent::__construct($crudService, $parameterBag);
	}

  public function visibilityFieldsReceita(DTO\Request $request, DTO\Response $response) {
    try {
        $row = (Array) $request->getRow();
        $sql = "SELECT P.CDFILIAL, P.IDQTLIQUBRUT, P.IDAPRERECE, P.IDGANHOCOCC, P.IDAPROVCOCC, P.IDPERDALIMP, P.IDAPROVLIMP,
                       P.IDTEMPPREP, P.IDBLOALTRELO, P.IDUTREQCOMPENTD, G.IDBLOPRATLOC, G.CDNVBLPRAT, P.IDUTDIASLIMSEG
                    FROM
                       PARAMGERAL G, PARAMPLANEJA P
                    WHERE
                       P.CDFILIAL = :P_CDFILIAL AND
                       P.CDPARPLANEJA = '1' AND
                       G.CDPARGERAL = '0001'";
                    
            $params = array(
                "P_CDFILIAL" => $row['CDFILCENTPME']
            );
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            
            $result = $std->fetchAll();

        $response->addDataset(new DataSet('result', $result[0]));
    } catch (\Exception $e) {
          throw new \Exception($e->getMessage(), $e->getCode());
      }
  }

  // public function returnVisibilityFields($cdfilial) {
  //       try {
  //           $sql = "SELECT
  //                     P.CDFILIAL,
  //                     P.IDQTLIQUBRUT,
  //                     P.IDAPRERECE,
  //                     P.IDGANHOCOCC,
  //                     P.IDAPROVCOCC,
  //                     P.IDPERDALIMP,
  //                     P.IDAPROVLIMP,
  //                     P.IDTEMPPREP,
  //                     P.IDBLOALTRELO,
  //                     P.IDUTREQCOMPENTD,   
  //                     G.IDBLOPRATLOC,
  //                     G.CDNVBLPRAT,
  //                     P.IDUTDIASLIMSEG
  //                   FROM
  //                     PARAMGERAL G, PARAMPLANEJA P
  //                   WHERE
  //                     P.CDFILIAL = :P_CDFILIAL AND
  //                     P.CDPARPLANEJA = '1' AND
  //                     G.CDPARGERAL = '0001'";
                    
  //           $params = array(
  //               "P_CDFILIAL" => $cdfilial
  //           );
  //           $std = $this->entityManager->getConnection()->prepare($sql);
  //           $std->execute($params);
            
  //           return $std->fetchAll();
            
  //       } catch (\Exception $e) {
  //           throw $e;
  //       }
  // }

  public function saveRecipeStandard(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = (Array) $request->getRow();

      $rows = $this->validRows($rows, 'recepadr');

      foreach ($rows as $row) {
        $this->insertRecipeStandard($row);        
      }
    
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  public function insertRecipeStandard($row) {
    if($row['__is_new']) {
      $query = "INSERT INTO RECEPADR (CDFILAUXPRAT, CDPRATO, CDPRODUTO, VRPEFATCORPR, SGUNIDADE, VRPEFATCOCPR, 
            QTPRODPRAT, HRTEMPPREPPR, QTUNIDESTPR, QTBRUTAPR, IDPRODBASEPR, CDAPRERECE, VRPEFATNUTPR, QTCALCNUTRPR, QTBRUTAPRUNR, IDQTDEFIXPR, QTLIMSEGPADR, IDUTCOMPENTDIRP, QTCOMPLPRODPADR, IDTPREQPRODPADR)
            VALUES (:CDFILAUXPRAT, :CDPRATO, :CDPRODUTO, :VRPEFATCORPR, :SGUNIDADE, :VRPEFATCOCPR, :QTPRODPRAT, :HRTEMPPREPPR, :QTUNIDESTPR, 
              :QTBRUTAPR, :IDPRODBASEPR, :CDAPRERECE, :VRPEFATNUTPR, :QTCALCNUTRPR, :QTBRUTAPRUNR, :IDQTDEFIXPR, :QTLIMSEGPADR, :IDUTCOMPENTDIRP, :QTCOMPLPRODPADR, :IDTPREQPRODPADR)";
    } else {
      $query = "UPDATE RECEPADR SET VRPEFATCORPR = :VRPEFATCORPR, SGUNIDADE = :SGUNIDADE, VRPEFATCOCPR = :VRPEFATCOCPR, QTPRODPRAT = :QTPRODPRAT, HRTEMPPREPPR = :HRTEMPPREPPR, QTUNIDESTPR = :QTUNIDESTPR, IDUTCOMPENTDIRP = :IDUTCOMPENTDIRP,
                QTBRUTAPR = :QTBRUTAPR, IDPRODBASEPR = :IDPRODBASEPR, CDAPRERECE = :CDAPRERECE, VRPEFATNUTPR = :VRPEFATNUTPR, QTCALCNUTRPR = :QTCALCNUTRPR, QTBRUTAPRUNR = :QTBRUTAPRUNR, IDQTDEFIXPR = :IDQTDEFIXPR, QTLIMSEGPADR = :QTLIMSEGPADR,
                QTCOMPLPRODPADR = :QTCOMPLPRODPADR, IDTPREQPRODPADR = :IDTPREQPRODPADR
                WHERE CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRODUTO = :CDPRODUTO";
    }  
        
    $params = array(
      'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
      'CDPRATO'       => $row['CDPRATO'],
      'CDPRODUTO'     => $row['CDPRODUTO'],
      'VRPEFATCORPR'  => $row['VRPEFATCORPR'],
      'SGUNIDADE'     => $row['SGUNIDADE'],
      'VRPEFATCOCPR'  => $row['VRPEFATCOCPR'],
      'QTPRODPRAT'    => $row['QTPRODPRAT'],
      'HRTEMPPREPPR'  => $row['HRTEMPPREPPR'],
      'QTUNIDESTPR'   => $row['QTUNIDESTPR'],
      'QTBRUTAPR'     => $row['QTBRUTAPR'],
      'IDPRODBASEPR'  => $row['IDPRODBASEPR'],
      'CDAPRERECE'    => $row['CDAPRERECE'],
      'VRPEFATNUTPR'  => $row['VRPEFATNUTPR'],
      'QTCALCNUTRPR'  => $row['QTCALCNUTRPR'],
      'QTBRUTAPRUNR'  => $row['QTBRUTAPRUNR'],
      'IDQTDEFIXPR'  => $row['IDQTDEFIXPR'],
      'QTLIMSEGPADR'  => $row['QTLIMSEGPADR'],
      'IDUTCOMPENTDIRP'  => $row['IDUTCOMPENTDIRP'],
      'QTCOMPLPRODPADR'  => $row['QTCOMPLPRODPADR'],
      'IDTPREQPRODPADR'  => $row['IDTPREQPRODPADR']
    );
    
    $this->connection->prepare($query)->execute($params);
  }
  
  public function saveRecipeTpCard(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = (Array) $request->getRow();
      $rows = $this->validRows($rows, 'recetpcard');
      
      foreach ($rows as $row) {
        $this->insertRecipeTpCard($row);
      }
    
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  public function insertRecipeTpCard($row) {
    if($row['__is_new']) {
      $query = "INSERT INTO RECETPCARD (CDFILAUXPRAT, CDPRATO, CDFILCENTPME, CDTIPOCARD, CDPRODUTO, VRPEFATCOCPRTP,
                    QTPRODPRATTP, VRPEFATCORPRTP, SGUNIDADE, HRTEMPPREPPRTP, QTUNIDESTPRTP, QTBRUTAPRTP,
                    IDPRODBASEPRTP, CDAPRERECE,VRPEFATNUTPRTP,QTCALCNUTRPRTP, QTBRUTAPRUNRTP, IDQTDEFIXPRTP)
                VALUES (:CDFILAUXPRAT, :CDPRATO, :CDFILCENTPME, :CDTIPOCARD, :CDPRODUTO, :VRPEFATCOCPRTP,
                    :QTPRODPRATTP, :VRPEFATCORPRTP, :SGUNIDADE, :HRTEMPPREPPRTP, :QTUNIDESTPRTP, :QTBRUTAPRTP,
                    :IDPRODBASEPRTP, :CDAPRERECE,:VRPEFATNUTPRTP,:QTCALCNUTRPRTP, :QTBRUTAPRUNRTP, :IDQTDEFIXPRTP)";
    } else {
      $query = "UPDATE RECETPCARD SET VRPEFATCORPRTP = :VRPEFATCORPRTP, SGUNIDADE = :SGUNIDADE, VRPEFATCOCPRTP = :VRPEFATCOCPRTP, QTPRODPRATTP = :QTPRODPRATTP, HRTEMPPREPPRTP = :HRTEMPPREPPRTP, QTUNIDESTPRTP = :QTUNIDESTPRTP, 
                QTBRUTAPRTP = :QTBRUTAPRTP, IDPRODBASEPRTP = :IDPRODBASEPRTP, CDAPRERECE = :CDAPRERECE, VRPEFATNUTPRTP = :VRPEFATNUTPRTP, QTCALCNUTRPRTP = :QTCALCNUTRPRTP, QTBRUTAPRUNRTP = :QTBRUTAPRUNRTP, IDQTDEFIXPRTP = :IDQTDEFIXPRTP
                WHERE CDFILCENTPME = :CDFILCENTPME AND CDTIPOCARD = :CDTIPOCARD AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRODUTO = :CDPRODUTO";
    }  
    $params = array(
      'CDFILCENTPME'  => $row['CDFILCENTPME'],
      'CDTIPOCARD'  => $row['CDTIPOCARD'],
      'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
      'CDPRATO'       => $row['CDPRATO'],
      'CDPRODUTO'     => $row['CDPRODUTO'],
      'VRPEFATCORPRTP'  => $row['VRPEFATCORPRTP'],
      'SGUNIDADE'     => $row['SGUNIDADE'],
      'VRPEFATCOCPRTP'  => $row['VRPEFATCOCPRTP'],
      'QTPRODPRATTP'    => $row['QTPRODPRATTP'],
      'HRTEMPPREPPRTP'  => $row['HRTEMPPREPPRTP'],
      'QTUNIDESTPRTP'   => $row['QTUNIDESTPRTP'],
      'QTBRUTAPRTP'     => $row['QTBRUTAPRTP'],
      'IDPRODBASEPRTP'  => $row['IDPRODBASEPRTP'],
      'CDAPRERECE'    => $row['CDAPRERECE'],
      'VRPEFATNUTPRTP'  => $row['VRPEFATNUTPRTP'],
      'QTCALCNUTRPRTP'  => $row['QTCALCNUTRPRTP'],
      'QTBRUTAPRUNRTP'  => $row['QTBRUTAPRUNRTP'],
      'IDQTDEFIXPRTP'   => $row['IDQTDEFIXPRTP']
    );
        
    $this->connection->prepare($query)->execute($params);
  }

  public function saveRecipeLocal(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = (Array) $request->getRow();
      $rows = $this->validRows($rows, 'receitaLocal');

      foreach ($rows as $row) {
        $this->insertRecipeLocal($row);
      }
    
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  public function insertRecipeLocal($row) {
    if($row['__is_new']) {
      $query = "INSERT INTO RECEFILI (CDFILAUXPRAT, CDPRATO, CDFILIAL, CDPRODUTO, SGUNIDADE, VRPEFATCOCLO, 
                  QTPRODRECLOC, VRPEFATCORLO, HRTEMPPREPLO, QTUNIDESTLO, QTBRUTALO, IDPRODBASELO, 
                  CDAPRERECE, VRPEFATNUTLO, QTCALCNUTRLO, QTBRUTALOUNR, IDQTDEFIXLO, QTLIMSEGFILI, IDUTCOMPENTDIRF, QTCOMPLPRODFILI, IDTPREQPRODFILI)
              VALUES (:CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :CDPRODUTO, :SGUNIDADE, :VRPEFATCOCLO, 
                  :QTPRODRECLOC, :VRPEFATCORLO, :HRTEMPPREPLO, :QTUNIDESTLO, :QTBRUTALO, 
                  :IDPRODBASELO, :CDAPRERECE, :VRPEFATNUTLO, :QTCALCNUTRLO, :QTBRUTALOUNR, :IDQTDEFIXLO, :QTLIMSEGFILI, :IDUTCOMPENTDIRF, :QTCOMPLPRODFILI, :IDTPREQPRODFILI)";
    } else {
      $query = "UPDATE RECEFILI SET VRPEFATCORLO = :VRPEFATCORLO, SGUNIDADE = :SGUNIDADE, VRPEFATCOCLO = :VRPEFATCOCLO, QTPRODRECLOC = :QTPRODRECLOC, HRTEMPPREPLO = :HRTEMPPREPLO, QTUNIDESTLO = :QTUNIDESTLO, 
                QTBRUTALO = :QTBRUTALO, IDPRODBASELO = :IDPRODBASELO, CDAPRERECE = :CDAPRERECE, VRPEFATNUTLO = :VRPEFATNUTLO, QTCALCNUTRLO = :QTCALCNUTRLO, QTBRUTALOUNR = :QTBRUTALOUNR, IDQTDEFIXLO = :IDQTDEFIXLO,
                QTLIMSEGFILI = :QTLIMSEGFILI, IDUTCOMPENTDIRF = :IDUTCOMPENTDIRF, QTCOMPLPRODFILI = :QTCOMPLPRODFILI, IDTPREQPRODFILI = :IDTPREQPRODFILI
                WHERE CDFILIAL = :CDFILIAL AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRODUTO = :CDPRODUTO";
    }  
        
    $params = array(
      'CDFILIAL'  => $row['CDFILIAL'],
      'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
      'CDPRATO'       => $row['CDPRATO'],
      'CDPRODUTO'     => $row['CDPRODUTO'],
      'VRPEFATCORLO'  => $row['VRPEFATCORLO'],
      'SGUNIDADE'     => $row['SGUNIDADE'],
      'VRPEFATCOCLO'  => $row['VRPEFATCOCLO'],
      'QTPRODRECLOC'  => $row['QTPRODRECLOC'],
      'HRTEMPPREPLO'  => $row['HRTEMPPREPLO'],
      'QTUNIDESTLO'   => $row['QTUNIDESTLO'],
      'QTBRUTALO'     => $row['QTBRUTALO'],
      'IDPRODBASELO'  => $row['IDPRODBASELO'],
      'CDAPRERECE'    => $row['CDAPRERECE'],
      'VRPEFATNUTLO'  => $row['VRPEFATNUTLO'],
      'QTCALCNUTRLO'  => $row['QTCALCNUTRLO'],
      'QTBRUTALOUNR'  => $row['QTBRUTALOUNR'],
      'IDQTDEFIXLO'  => $row['IDQTDEFIXLO'],
      'QTLIMSEGFILI'  => $row['QTLIMSEGFILI'],
      'IDUTCOMPENTDIRF'  => $row['IDUTCOMPENTDIRF'],
      'QTCOMPLPRODFILI'  => $row['QTCOMPLPRODFILI'],
      'IDTPREQPRODFILI'  => $row['IDTPREQPRODFILI']
    );
    $this->connection->prepare($query)->execute($params);
  }
  
  public function saveRecipeRegion(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = (Array) $request->getRow();
      $rows['NRRECEREGIAO'] = $this->newCode->getCodeByTableName('RECEREGIAO', $this->environment->getNrOrg());
      // $rows = $this->validRows($rows, 'receitaRegiao');
      foreach ($rows as $row) {
        $this->insertRecipeRegion($row);
      }
    
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  private function insertRecipeRegion($row) {
    if($row['__is_new']) $query = "INSERT INTO RECEREGIAO (NRRECEREGIAO, CDFILAUXPRAT, CDPRATO, CDREGIAO, NRPADRAOQUALIDADE, CDPRODUTO, VRPEFATCOCREGIAO, VRPEFATCORREGIAO, SGUNIDADE, QTBRUTAREGIAOUNR, IDPRODBASEREGIAO, IDQTDEFIXREGIAO) VALUES (:NRRECEREGIAO, :CDFILAUXPRAT, :CDPRATO, :CDREGIAO, :NRPADRAOQUALIDADE, :CDPRODUTO, :VRPEFATCOCREGIAO, :VRPEFATCORREGIAO, :SGUNIDADE, :QTBRUTAREGIAOUNR, :IDPRODBASEREGIAO, :IDQTDEFIXREGIAO)";
    else $query = "UPDATE RECEFILI SET VRPEFATCORLO = :VRPEFATCORLO, SGUNIDADE = :SGUNIDADE, VRPEFATCOCLO = :VRPEFATCOCLO, QTPRODRECLOC = :QTPRODRECLOC, HRTEMPPREPLO = :HRTEMPPREPLO, QTUNIDESTLO = :QTUNIDESTLO, 
                QTBRUTALO = :QTBRUTALO, IDPRODBASELO = :IDPRODBASELO, CDAPRERECE = :CDAPRERECE, VRPEFATNUTLO = :VRPEFATNUTLO, QTCALCNUTRLO = :QTCALCNUTRLO, QTBRUTALOUNR = :QTBRUTALOUNR, IDQTDEFIXLO = :IDQTDEFIXLO,
                QTLIMSEGFILI = :QTLIMSEGFILI, IDUTCOMPENTDIRF = :IDUTCOMPENTDIRF, QTCOMPLPRODFILI = :QTCOMPLPRODFILI, IDTPREQPRODFILI = :IDTPREQPRODFILI
                WHERE CDFILIAL = :CDFILIAL AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRODUTO = :CDPRODUTO";
        
    $params = array(
      'NRRECEREGIAO'      => $row['NRRECEREGIAO'],
      'CDFILAUXPRAT'      => $row['CDFILAUXPRAT'],
      'CDPRATO'           => $row['CDPRATO'],
      'CDREGIAO'          => $row['CDREGIAO'],
      'NRPADRAOQUALIDADE' => $row['NRPADRAOQUALIDADE'],
      'CDPRODUTO'         => $row['CDPRODUTO'],
      'VRPEFATCOCREGIAO'  => $row['VRPEFATCOCREGIAO'],
      'VRPEFATCORREGIAO'  => $row['VRPEFATCORREGIAO'],
      'SGUNIDADE'         => $row['SGUNIDADE'],
      'QTBRUTAREGIAOUNR'  => $row['QTBRUTAREGIAOUNR'],
      'IDPRODBASEREGIAO'  => $row['IDPRODBASEREGIAO'],
      'IDQTDEFIXREGIA'    => $row['IDQTDEFIXREGIA']
    );
    // var_dump($this->connection->prepare($query)->execute($params));die;
    $this->connection->prepare($query)->execute($params);
  }

  private function insertRecipeQual($row) {
    if($row['__is_new']) $query = "INSERT INTO RECEPADQUALI (NRRECEPADQUALI, CDFILAUXPRAT, CDPRATO, NRPADRAOQUALIDADE, CDPRODUTO, QTPRODRECPADQ, VRPEFATCOCPADQ, VRPEFATCOCLOAP, VRPEFATCORPADQ, VRPEFATCORLOAP, SGUNIDADE, QTBRUTAPADQUNR, IDPRODBASEPADQ, IDQTDEFIXPADQ) 
      VALUES (:NRRECEPADQUALI, :CDFILAUXPRAT, :CDPRATO, :NRPADRAOQUALIDADE, :CDPRODUTO, :QTPRODRECPADQ, :VRPEFATCOCPADQ, :VRPEFATCOCLOAP, :VRPEFATCORPADQ, :VRPEFATCORLOAP, :SGUNIDADE, :QTBRUTAPADQUNR, :IDPRODBASEPADQ, :IDQTDEFIXPADQ)";
    else $query = "UPDATE RECEPADQUALI SET QTPRODRECPADQ = :QTPRODRECPADQ, VRPEFATCOCPADQ = :VRPEFATCOCPADQ, VRPEFATCOCLOAP = :VRPEFATCOCLOAP, VRPEFATCORPADQ = :VRPEFATCORPADQ, VRPEFATCORLOAP = :VRPEFATCORLOAP, SGUNIDADE = :SGUNIDADE, QTBRUTAPADQUNR = :QTBRUTAPADQUNR, IDPRODBASEPADQ = :IDPRODBASEPADQ, IDQTDEFIXPADQ = :IDQTDEFIXPADQ
                WHERE NRRECEPADQUALI = :NRRECEPADQUALI AND NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE AND CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRODUTO = :CDPRODUTO";
        
    $params = array(
      'NRRECEPADQUALI'    => $row['NRRECEPADQUALI'],
      'CDFILAUXPRAT'      => $row['CDFILAUXPRAT'],
      'CDPRATO'           => $row['CDPRATO'],
      'NRPADRAOQUALIDADE' => $row['NRPADRAOQUALIDADE'],
      'CDPRODUTO'         => $row['CDPRODUTO'],
      'QTPRODRECPADQ'     => $row['QTPRODRECPADQ'],
      'VRPEFATCOCPADQ'    => $row['VRPEFATCOCPADQ'],
      'VRPEFATCOCLOAP'    => $row['VRPEFATCOCLOAP'],
      'VRPEFATCORPADQ'    => $row['VRPEFATCORPADQ'],
      'VRPEFATCORLOAP'    => $row['VRPEFATCORLOAP'],
      'QTBRUTAPADQUNR'    => $row['QTBRUTAPADQUNR'],
      'IDPRODBASEPADQ'    => $row['IDPRODBASEPADQ'],
      'IDQTDEFIXPADQ'     => $row['IDQTDEFIXPADQ'],
      'SGUNIDADE'         => $row['SGUNIDADE']
    );
    
    $this->connection->prepare($query)->execute($params);
  }

  public function saveRecipeService(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $rows = (Array) $request->getRow();
      $rows = $this->validRows($rows, 'receitaServico');
      
      foreach ($rows as $row) {
        $this->insertRecipeService($row);
      }
    
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  public function insertRecipeService($row) {
    if($row['__is_new']) {
      $query = "INSERT INTO RECESERV (CDFILAUXPRAT, CDPRATO, CDFILIAL, CDSERVFILI, CDPRODUTO, VRPEFATCOCSE, 
                  QTPRODRECSER, VRPEFATCORSE, SGUNIDADE, HRTEMPPREPSE, QTUNIDESTSE, QTBRUTASE, 
                  IDPRODBASESE, CDAPRERECE,VRPEFATNUTSE,QTCALCNUTRSE, QTBRUTASEUNR, IDQTDEFIXSE, QTLIMSEGSERV, IDUTCOMPENTDIRS, QTCOMPLPRODSERV, IDTPREQPRODSERV)
              VALUES (:CDFILAUXPRAT, :CDPRATO, :CDFILIAL, :CDSERVFILI, :CDPRODUTO, :VRPEFATCOCSE, 
                  :QTPRODRECSER, :VRPEFATCORSE, :SGUNIDADE, :HRTEMPPREPSE, :QTUNIDESTSE, 
                  :QTBRUTASE, :IDPRODBASESE, :CDAPRERECE, :VRPEFATNUTSE, :QTCALCNUTRSE, :QTBRUTASEUNR, :IDQTDEFIXSE, :QTLIMSEGSERV, :IDUTCOMPENTDIRS, :QTCOMPLPRODSERV, :IDTPREQPRODSERV)
              ";
    } else {
      $query = "UPDATE RECESERV SET VRPEFATCORSE = :VRPEFATCORSE, SGUNIDADE = :SGUNIDADE, VRPEFATCOCSE = :VRPEFATCOCSE, QTPRODRECSER = :QTPRODRECSER, HRTEMPPREPSE = :HRTEMPPREPSE, QTUNIDESTSE = :QTUNIDESTSE, 
                QTBRUTASE = :QTBRUTASE, IDPRODBASESE = :IDPRODBASESE, CDAPRERECE = :CDAPRERECE, VRPEFATNUTSE = :VRPEFATNUTSE, QTCALCNUTRSE = :QTCALCNUTRSE, QTBRUTASEUNR = :QTBRUTASEUNR, IDQTDEFIXSE = :IDQTDEFIXSE,
                QTLIMSEGSERV = :QTLIMSEGSERV, IDUTCOMPENTDIRS = :IDUTCOMPENTDIRS, QTCOMPLPRODSERV = :QTCOMPLPRODSERV, IDTPREQPRODSERV = :IDTPREQPRODSERV
                WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND  CDPRATO = :CDPRATO AND CDFILAUXPRAT = :CDFILAUXPRAT AND CDPRODUTO = :CDPRODUTO";
    }  
        
    $params = array(
      'CDFILIAL'  => $row['CDFILIAL'],
      'CDSERVFILI'  => $row['CDSERVFILI'],
      'CDFILAUXPRAT'  => $row['CDFILAUXPRAT'],
      'CDPRATO'       => $row['CDPRATO'],
      'CDPRODUTO'     => $row['CDPRODUTO'],
      'VRPEFATCORSE'  => $row['VRPEFATCORSE'],
      'SGUNIDADE'     => $row['SGUNIDADE'],
      'VRPEFATCOCSE'  => $row['VRPEFATCOCSE'],
      'QTPRODRECSER'    => $row['QTPRODRECSER'],
      'HRTEMPPREPSE'  => $row['HRTEMPPREPSE'],
      'QTUNIDESTSE'   => $row['QTUNIDESTSE'],
      'QTBRUTASE'     => $row['QTBRUTASE'],
      'IDPRODBASESE'  => $row['IDPRODBASESE'],
      'CDAPRERECE'    => $row['CDAPRERECE'],
      'VRPEFATNUTSE'  => $row['VRPEFATNUTSE'],
      'QTCALCNUTRSE'  => $row['QTCALCNUTRSE'],
      'QTBRUTASEUNR'  => $row['QTBRUTASEUNR'],
      'IDQTDEFIXSE'  => $row['IDQTDEFIXSE'],
      'QTLIMSEGSERV'  => $row['QTLIMSEGSERV'],
      'IDUTCOMPENTDIRS'  => $row['IDUTCOMPENTDIRS'],
      'QTCOMPLPRODSERV'  => $row['QTCOMPLPRODSERV'],
      'IDTPREQPRODSERV'  => $row['IDTPREQPRODSERV']
    );

    $this->connection->prepare($query)->execute($params);
  }

  private function validRows($rows, $dataSourceName) {
    $dataSet = new DataSet($dataSourceName, $rows);
    $request = new DTO\Request\DataSet($dataSet, "", "", "");
    $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
    return $rowsToSave;
  }

  public function saveImportRecipe(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();

      $params = (Array) $request->getRow();
      $rows = $params[0];

      $prodDest = $params[1];

      $firstRow = $rows[0];
      $dest = $firstRow['RECIPEDEST'];
      if($firstRow['IMPORTTYPE'] == 'SOBREPOR') {
        $this->deleteRecipe($firstRow);
        $this->deletePlate($firstRow);
        $this->updatePlate($firstRow, $dest);
      } else {
        $filteredRows = [];
        foreach($rows as $row) {
          $valid = true;
          foreach($prodDest as $prod){
            if($row['CDPRODUTO'] == $prod['CDPRODUTO']) $valid = false;
          }
          if($valid) array_push($filteredRows, $row);
        }

        $rows = $filteredRows;
      }

      if($dest == 'PADRAO') {
        $typeFieldVR = 'PR';
        $typeFieldQT = 'PRAT';
        $typeFieldSEG = 'PADR';
      } else if($dest == 'TIPOCARD') {
        $typeFieldVR = 'PRTP';
        $typeFieldQT = 'PRATTP';
        $typeFieldSEG = ' ';
      } else if($dest == 'LOCAL'){
        $typeFieldVR = 'LO';
        $typeFieldQT = 'RECLOC';
        $typeFieldSEG = 'FILI';
      } else {
        $typeFieldVR = 'SE';
        $typeFieldQT = 'RECSER';
        $typeFieldSEG = 'SERV';
      }

      foreach ($rows as $row) {
        $row['__is_new'] = true;
        $this->setProperties($row, $typeFieldVR, $typeFieldQT, $typeFieldSEG);
        if($dest == 'PADRAO') $this->insertRecipeStandard($row);
        else if($dest == 'TIPOCARD') $this->insertRecipeTpCard($row);
        else if($dest == 'LOCAL') $this->insertRecipeLocal($row);
        else $this->insertRecipeService($row);
      }
    
      $this->connection->commit();
      $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new DTO\Response\Error($e->getMessage(), 1));
    }
  }

  private function setProperties(&$row, $typeFieldVR, $typeFieldQT, $typeFieldSEG) {
    $row['VRPEFATCOC'.$typeFieldVR] = $row['VRPEFATCOC'];
    $row['VRPEFATCOC'.$typeFieldVR.'AP'] = $row['VRPEFATCOCAP'];
    
    $row['VRPEFATCOR'.$typeFieldVR] = $row['VRPEFATCOR'];
    $row['VRPEFATCOR'.$typeFieldVR.'AP'] = $row['VRPEFATCORAP'];

    $row['QTBRUTA'.$typeFieldVR] = $row['QTBRUTA'];
    $row['QTPROD'.$typeFieldQT] = $row['QTPRODPRAT'];

    $row['QTBRUTA'.$typeFieldVR.'UNR'] = $row['QTBRUTAUNR'];
    if ($typeFieldVR === 'PRTP') $row['QTBRUTAPRUNRTP'] = $row['QTBRUTAUNR'];

    $row['QTCALCNUTR'.$typeFieldVR] = $row['QTCALCNUTR'];
    $row['QTUNIDEST'.$typeFieldVR] = $row['QTUNIDEST'];

    $row['HRTEMPPREP'.$typeFieldVR] = $row['HRTEMPPREP'];
    $row['IDPRODBASE'.$typeFieldVR] = $row['IDPRODBASE'];
    $row['VRPEFATNUT'.$typeFieldVR] = $row['VRPEFATNUT'];
    $row['IDQTDEFIX'.$typeFieldVR] = $row['IDQTDEFIX'];

    
    $row['QTLIMSEG'.$typeFieldSEG] = empty($row['QTLIMSEG']) ? null : $row['QTLIMSEG'];
    $row['QTCOMPLPROD'.$typeFieldSEG] = empty($row['QTCOMPLPROD']) ? 0 : $row['QTCOMPLPROD'];
    $row['IDTPREQPROD'.$typeFieldSEG] = empty($row['IDTPREQPROD']) ? 'P' : $row['IDTPREQPROD'];
    $row['IDUTCOMPENTDIR'.substr($typeFieldSEG,0,1)] = empty($row['IDUTCOMPENTDIR']) ? 'N' : $row['IDUTCOMPENTDIR'];
  }

  private function deleteRecipe($firstRow) {
    $query = 'DELETE FROM RECEPADR WHERE CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
    $params = array(
      'P_CDPRATDEST'      => $firstRow['CDPRATO'],
      'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT']
    );

    if($firstRow['RECIPEDEST'] == 'TIPOCARD'){
      $query = 'DELETE FROM RECETPCARD WHERE CDFILCENTPME = :P_CDFILCENTPMEDEST AND CDTIPOCARD = :P_CDTIPOCARDDEST AND CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
      $params = array(
        'P_CDPRATDEST'      => $firstRow['CDPRATO'],
        'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT'],
        'P_CDFILCENTPMEDEST'=> $firstRow['CDFILCENTPME'],
        'P_CDTIPOCARDDEST'  => $firstRow['CDTIPOCARD']
      );
    }
    else if($firstRow['RECIPEDEST'] == 'LOCAL'){
      $query = 'DELETE FROM RECEFILI WHERE CDFILIAL = :P_CDFILIALDEST AND CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
      $params = array(
        'P_CDPRATDEST'      => $firstRow['CDPRATO'],
        'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT'],
        'P_CDFILIALDEST'    => $firstRow['CDFILIAL']
      );
    }
    else if($firstRow['RECIPEDEST'] == 'SERVICO'){
      $query = 'DELETE FROM RECESERV WHERE CDFILIAL = :P_CDFILIALDEST AND CDSERVFILI = :P_CDSERVFILIDEST AND CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
      $params = array(
        'P_CDPRATDEST'      => $firstRow['CDPRATO'],
        'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT'],
        'P_CDFILIALDEST'    => $firstRow['CDFILIAL'],
        'P_CDSERVFILIDEST'  => $firstRow['CDSERVFILI']
      );
    }
    $this->connection->prepare($query)->execute($params);
  }
  
  private function deletePlate($firstRow) {
    $query = 'DELETE FROM PRATPADR WHERE CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
    $params = array(
      'P_CDPRATDEST'      => $firstRow['CDPRATO'],
      'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT']
    );

    if($firstRow['RECIPEDEST'] == 'TIPOCARD'){
      $query = 'DELETE FROM PRATTPCARD WHERE CDFILCENTPME = :P_CDFILCENTPMEDEST AND CDTIPOCARD = :P_CDTIPOCARDDEST AND CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
      $params = array(
        'P_CDPRATDEST'      => $firstRow['CDPRATO'],
        'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT'],
        'P_CDFILCENTPMEDEST'=> $firstRow['CDFILCENTPME'],
        'P_CDTIPOCARDDEST'  => $firstRow['CDTIPOCARD']
      );
    }
    else if($firstRow['RECIPEDEST'] == 'LOCAL'){
      $query = 'DELETE FROM PRATFILI WHERE CDFILIAL = :P_CDFILIALDEST AND CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
      $params = array(
        'P_CDPRATDEST'      => $firstRow['CDPRATO'],
        'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT'],
        'P_CDFILIALDEST'    => $firstRow['CDFILIAL']
      );
    }
    else if($firstRow['RECIPEDEST'] == 'SERVICO'){
      $query = 'DELETE FROM PRATSERV WHERE CDFILIAL = :P_CDFILIALDEST AND CDSERVFILI = :P_CDSERVFILIDEST AND CDPRATO = :P_CDPRATDEST AND CDFILAUXPRAT = :P_CDFILAUXDEST';
      $params = array(
        'P_CDPRATDEST'      => $firstRow['CDPRATO'],
        'P_CDFILAUXDEST'    => $firstRow['CDFILAUXPRAT'],
        'P_CDFILIALDEST'    => $firstRow['CDFILIAL'],
        'P_CDSERVFILIDEST'  => $firstRow['CDSERVFILI']
      );
    }
    $this->connection->prepare($query)->execute($params);
  }
  
  private function updatePlate($firstRow, $dest) {
    $origem = $firstRow['ORIGEM'];
    
    $query = "SELECT :P_CDPRATDEST CDPRATO, :P_CDFILAUXDEST CDFILAUXPRAT, :P_CDFILCENTPMEDEST CDFILCENTPME, :P_CDTIPOCARDDEST CDTIPOCARD, :P_CDFILIDEST CDFILIAL, :P_CDSERVDEST CDSERVFILI, NRCORPRIPRAT NRCORPRI, NRCORSEQPRAT NRCORSEQ, NRCORAUXPRAT NRCORAUX, QTPADRAOPRAT QTPADRAO, QTCONTAIPRAT QTCONTAI,
                  HRVALCONPRAT HRVALCON, NMFANTASIAPR NMFANTASIA, QTCARBOPRPAD QTCARBOPR, QTLIPIDPRPAD QTLIPIDPR, QTPROTEPRPAD QTPROTEPR, QTCALORPRPAD QTCALORPR, QTALCOOPRPAD QTALCOOPR, VRPEMINPADR VRPEMIN, QTAMOSTRPADR QTAMOSTR
              FROM PRATPADR WHERE (CDPRATO = :P_CDPRATORIG) AND (CDFILAUXPRAT  = :P_CDFILAUXORIG) AND :P_IDTIPORECE = 'PADRAO'
              UNION ALL
              SELECT :P_CDPRATDEST CDPRATO, :P_CDFILAUXDEST CDFILAUXPRAT, :P_CDFILCENTPMEDEST CDFILCENTPME, :P_CDTIPOCARDDEST CDTIPOCARD, :P_CDFILIDEST CDFILIAL, :P_CDSERVDEST CDSERVFILI, NRCORPRIPRATTP NRCORPRI, NRCORSEQPRATTP NRCORSEQ, NRCORAUXPRATTP NRCORAUX, QTPADRAOPRATTP QTPADRAO, QTCONTAIPRATTP QTCONTAI,
                  HRVALCONPRATTP HRVALCON, NMFANTASIAPRTP NMFANTASIA, QTCARBOPRPADTP QTCARBOPR, QTLIPIDPRPADTP QTLIPIDPR, QTPROTEPRPADTP QTPROTEPR, QTCALORPRPADTP QTCALORPR, QTALCOOPRPADTP QTALCOOPR, VRPEMINPADRTP VRPEMIN, QTAMOSTRPADRTP QTAMOSTR
              FROM PRATTPCARD WHERE (CDPRATO = :P_CDPRATORIG) AND (CDFILAUXPRAT = :P_CDFILAUXORIG) AND (CDFILCENTPME = :P_CDFILCENTPMEORIG) AND (CDTIPOCARD = :P_CDTIPOCARDORIG) AND :P_IDTIPORECE = 'TIPOCARD'
              UNION ALL
              SELECT :P_CDPRATDEST CDPRATO, :P_CDFILAUXDEST CDFILAUXPRAT, :P_CDFILCENTPMEDEST CDFILCENTPME, :P_CDTIPOCARDDEST CDTIPOCARD, :P_CDFILIDEST CDFILIAL, :P_CDSERVDEST CDSERVFILI, NRCORPRIFILI NRCORPRI, NRCORSEQFILI NRCORSEQ, NRCORAUXFILI NRCORAUX, QTPADRAOFILI QTPADRAO, QTCONTAIFILI QTCONTAI,
                  HRVALCONFILI HRVALCON, NMFANTASIAPF NMFANTASIA, QTCARBOPRFIL QTCARBOPR, QTLIPIDPRFIL QTLIPIDPR, QTPROTEPRFIL QTPROTEPR, QTCALORPRFIL QTCALORPR, QTALCOOPRFIL QTALCOOPR, VRPEMINFILI VRPEMIN, QTAMOSTRFILI QTAMOSTR
              FROM PRATFILI WHERE (CDPRATO = :P_CDPRATORIG) AND (CDFILAUXPRAT = :P_CDFILAUXORIG) AND (CDFILIAL = :P_CDFILIORIG) AND :P_IDTIPORECE = 'LOCAL'
              UNION ALL
              SELECT :P_CDPRATDEST CDPRATO, :P_CDFILAUXDEST CDFILAUXPRAT, :P_CDFILCENTPMEDEST CDFILCENTPME, :P_CDTIPOCARDDEST CDTIPOCARD, :P_CDFILIDEST CDFILIAL, :P_CDSERVDEST CDSERVFILI, NRCORPRISERV NRCORPRI, NRCORSEQSERV NRCORSEQ, NRCORAUXSERV NRCORAUX, QTPADRAOSERV QTPADRAO, QTCONTAISERV QTCONTAI,
                  HRVALCONSERV HRVALCON, NMFANTASIASE NMFANTASIA, QTCARBOPRSER QTCARBOPR, QTLIPIDPRSER QTLIPIDPR, QTPROTEPRSER QTPROTEPR, QTCALORPRSER QTCALORPR, QTALCOOPRSER QTALCOOPR, VRPEMINSERV VRPEMIN, QTAMOSTRSERV QTAMOSTR
              FROM PRATSERV WHERE (CDPRATO = :P_CDPRATORIG) AND (CDFILAUXPRAT = :P_CDFILAUXORIG) AND (CDFILIAL = :P_CDFILIORIG) AND (CDSERVFILI = :P_CDSERVORIG) AND :P_IDTIPORECE = 'SERVICO'";


    $params = array(
      'P_CDPRATORIG'  => $origem['CDPRATO'],
      'P_CDFILAUXORIG'=> $origem['CDFILAUXPRAT'],
      'P_CDFILCENTPMEORIG' => $origem['CDFILCENTPME'],
      'P_CDTIPOCARDORIG'  => $origem['CDTIPOCARD'],
      'P_CDFILIORIG' => $origem['CDFILIAL'],
      'P_CDSERVORIG'  => $origem['CDSERVFILI'],
      'P_CDPRATDEST'  => $firstRow['CDPRATO'],
      'P_CDFILAUXDEST'=> $firstRow['CDFILAUXPRAT'],
      'P_CDFILCENTPMEDEST' => $firstRow['CDFILCENTPME'],
      'P_CDTIPOCARDDEST'  => $firstRow['CDTIPOCARD'],
      'P_CDFILIDEST' => $firstRow['CDFILIAL'],
      'P_CDSERVDEST'  => $firstRow['CDSERVFILI'],
      'P_IDTIPORECE'  => $firstRow['TYPE'],
    );
    
    $result = $this->connection->fetchAll($query, $params);
    if(!empty($result)) $this->insertPlateImport($result, $dest);
  }

  private function insertPlateImport($result, $dest){
    $params = array(
      'NRCORPRI' => $result[0]['NRCORPRI'],
      'NRCORSEQ' => $result[0]['NRCORSEQ'],
      'NRCORAUX' => $result[0]['NRCORAUX'],
      'QTPADRAO' => $result[0]['QTPADRAO'],
      'QTCONTAI' => $result[0]['QTCONTAI'],
      'HRVALCON' => $result[0]['HRVALCON'],
      'NMFANTASIA' => $result[0]['NMFANTASIA'],
      'QTCARBOPR' => $result[0]['QTCARBOPR'],
      'QTLIPIDPR' => $result[0]['QTLIPIDPR'],
      'QTPROTEPR' => $result[0]['QTPROTEPR'],
      'QTCALORPR' => $result[0]['QTCALORPR'],
      'QTALCOOPR' => $result[0]['QTALCOOPR'],
      'VRPEMIN' => $result[0]['VRPEMIN'],
      'QTAMOSTR' => $result[0]['QTAMOSTR']
    );    

    if ($dest == 'PADRAO'){
      $insert = "INSERT INTO PRATPADR (CDFILAUXPRAT, CDPRATO, NRCORPRIPRAT, NRCORSEQPRAT, NRCORAUXPRAT, QTPADRAOPRAT, QTCONTAIPRAT,
                    HRVALCONPRAT, NMFANTASIAPR,QTCARBOPRPAD, QTLIPIDPRPAD, QTPROTEPRPAD, QTCALORPRPAD, QTALCOOPRPAD, VRPEMINPADR, QTAMOSTRPADR)
                 VALUES (:CDFILAUXPRAT, :CDPRATO, :NRCORPRI, :NRCORSEQ, :NRCORAUX, :QTPADRAO, :QTCONTAI,
                    :HRVALCON, :NMFANTASIA, :QTCARBOPR, :QTLIPIDPR, :QTPROTEPR, :QTCALORPR, :QTALCOOPR, :VRPEMIN, :QTAMOSTR)";
      
      $paramsDest = array(
        'CDFILAUXPRAT' => $result[0]['CDFILAUXPRAT'],
        'CDPRATO' => $result[0]['CDPRATO']
      );
    } else if ($dest == 'TIPOCARD'){
      $insert = "INSERT INTO PRATTPCARD (CDFILCENTPME, CDTIPOCARD, CDFILAUXPRAT, CDPRATO, NRCORPRIPRATTP, NRCORSEQPRATTP, NRCORAUXPRATTP,QTPADRAOPRATTP, QTCONTAIPRATTP,
                    HRVALCONPRATTP, NMFANTASIAPRTP, QTCARBOPRPADTP, QTLIPIDPRPADTP, QTPROTEPRPADTP, QTCALORPRPADTP, QTALCOOPRPADTP,VRPEMINPADRTP, QTAMOSTRPADRTP)
                    VALUES ( :CDFILCENTPME, :CDTIPOCARD, :CDFILAUXPRAT, :CDPRATO, :NRCORPRI, :NRCORSEQ, :NRCORAUX, :QTPADRAO, :QTCONTAI,
                    :HRVALCON, :NMFANTASIA, :QTCARBOPR, :QTLIPIDPR, :QTPROTEPR, :QTCALORPR, :QTALCOOPR, :VRPEMIN, :QTAMOSTR)";
      $paramsDest = array(
        'CDFILCENTPME' => $result[0]['CDFILCENTPME'],
        'CDTIPOCARD' => $result[0]['CDTIPOCARD'],
        'CDFILAUXPRAT' => $result[0]['CDFILAUXPRAT'],
        'CDPRATO' => $result[0]['CDPRATO']
      );
    } else if ($dest == 'LOCAL'){
      $insert = "INSERT INTO PRATFILI (CDFILIAL, CDFILAUXPRAT, CDPRATO, NRCORPRIFILI, NRCORSEQFILI, NRCORAUXFILI, QTPADRAOFILI, QTCONTAIFILI,
                    HRVALCONFILI, NMFANTASIAPF, QTCARBOPRFIL, QTLIPIDPRFIL, QTPROTEPRFIL, QTCALORPRFIL, QTALCOOPRFIL, VRPEMINFILI, QTAMOSTRFILI)
                    VALUES (:CDFILIAL, :CDFILAUXPRAT, :CDPRATO, :NRCORPRI, :NRCORSEQ, :NRCORAUX, :QTPADRAO, :QTCONTAI,
                    :HRVALCON, :NMFANTASIA, :QTCARBOPR, :QTLIPIDPR, :QTPROTEPR, :QTCALORPR, :QTALCOOPR, :VRPEMIN, :QTAMOSTR)";
      $paramsDest = array(
        'CDFILIAL' => $result[0]['CDFILIAL'],
        'CDFILAUXPRAT' => $result[0]['CDFILAUXPRAT'],
        'CDPRATO' => $result[0]['CDPRATO']
      );
    } else {
      $insert = "INSERT INTO PRATSERV (CDPRATO, CDFILAUXPRAT, CDFILIAL, CDSERVFILI, NRCORPRISERV, NRCORSEQSERV, NRCORAUXSERV, QTPADRAOSERV, QTCONTAISERV,
                HRVALCONSERV, NMFANTASIASE, QTCARBOPRSER, QTLIPIDPRSER, QTPROTEPRSER, QTCALORPRSER, QTALCOOPRSER, VRPEMINSERV, QTAMOSTRSERV)
                VALUES ( :CDPRATO, :CDFILAUXPRAT, :CDFILIAL, :CDSERVFILI, :NRCORPRI, :NRCORSEQ, :NRCORAUX, :QTPADRAO, :QTCONTAI,
                :HRVALCON, :NMFANTASIA, :QTCARBOPR, :QTLIPIDPR, :QTPROTEPR, :QTCALORPR, :QTALCOOPR, :VRPEMIN, :QTAMOSTR)";
      $paramsDest = array(
        'CDPRATO' => $result[0]['CDPRATO'],  
        'CDFILAUXPRAT' => $result[0]['CDFILAUXPRAT'],
        'CDFILIAL' => $result[0]['CDFILIAL'],
        'CDSERVFILI' => $result[0]['CDSERVFILI']
      );
    }

    $params = array_merge($paramsDest, $params);
    $this->connection->prepare($insert)->execute($params);
  }
  
  public function returnReceitaPadrao(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();

	    $plate = $parameters['plate'];
	    $filAux = $parameters['filAux'];
	    $dataSourceName = 'receitaPadrao';
	    
	    // condição: CDPRATO, CDFILAUXPRAT
	    
	    $conditions = array(
	        array(
	            'columnName' => 'CDPRATO',
	            'operator' => '=',
	            'value' => $plate
	        ),
	        array(
	            'columnName' => 'CDFILAUXPRAT',
	            'operator' => '=',
	            'value' => $filAux
	        )
	    );	    
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    $response->addDataSet(new DataSet("columnsToReturn", $result));
	}
	
	public function returnReceitaLocal(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $plate = $parameters['plate'];
	    $filAux = $parameters['filAux'];
	    $cdfilial = $parameters['cdfilial'];
	    $dataSourceName = 'receitaLocal';
	    
	    // condição: CDPRATO, CDFILAUXPRAT, CDFILIAL
	    
	    $conditions = array(
	        array(
	            'columnName' => 'CDPRATO',
	            'operator' => '=',
	            'value' => $plate
	        ),
	        array(
	            'columnName' => 'CDFILAUXPRAT',
	            'operator' => '=',
	            'value' => $filAux
	        ),
	        array(
	            'columnName' => 'CDFILIAL',
	            'operator' => '=',
	            'value' => $cdfilial
	        )
	    );
    
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    $response->addDataSet(new DataSet("columnsToReturn", $result));
	}
	
	public function returnReceitaServico(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $plate = $parameters['plate'];
	    $filAux = $parameters['filAux'];
	    $cdfilial = $parameters['cdfilial'];
	    $cdserv = $parameters['cdserv'];
	    $dataSourceName = 'receitaServico';
	    
		// condição: CDPRATO, CDFILAUXPRAT, CDFILIAL, CDSERVFILI
	   
	    $conditions = array(
	        array(
	            'columnName' => 'CDPRATO',
	            'operator' => '=',
	            'value' => $plate
	        ),
	        array(
	            'columnName' => 'CDFILAUXPRAT',
	            'operator' => '=',
	            'value' => $filAux
	        ),
	        array(
	            'columnName' => 'CDFILIAL',
	            'operator' => '=',
	            'value' => $cdfilial
	        ),
	        array(
	            'columnName' => 'CDSERVFILI',
	            'operator' => '=',
	            'value' => $cdserv
	        )
	    );

	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    $response->addDataSet(new DataSet("columnsToReturn", $result));
	}
	
	public function returnReceitaTpCard(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $plate = $parameters['plate'];
	    $filAux = $parameters['filAux'];
	    $tipocard = $parameters['tipocard'];
	    $central = $parameters['central'];
	    $dataSourceName = 'receitaTpCard';
	    
	    // condição: CDPRATO, CDFILAUXPRAT, CDFILCENTPME, CDTIPOCARD

	    $conditions = array(
	        array(
	            'columnName' => 'P_CDPRATO',
	            'operator' => '=',
	            'value' => $plate
	        ),
	        array(
	            'columnName' => 'P_CDFILAUXPRAT',
	            'operator' => '=',
	            'value' => $filAux
	        ),
	        array(
	            'columnName' => 'P_CDFILCENTPME',
	            'operator' => '=',
	            'value' => $central
	        ),
	        array(
	            'columnName' => 'P_CDTIPOCARD',
	            'operator' => '=',
	            'value' => $tipocard
	        )
	    );
	    
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    $response->addDataSet(new DataSet("columnsToReturn", $result));
	}
	
	public function returnReceitaPadraoQualidade(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $plate = $parameters['plate'];
	    $filAux = $parameters['filAux'];
	    $dataSourceName = 'receitaPadraoQualidade';
	  
	   // condição: CDPRATO, CDFILAUXPRAT, NRPADRAOQUALIDADE

	    $conditions = array(
	        array(
	            'columnName' => 'CDPRATO',
	            'operator' => '=',
	            'value' => $plate
	        ),
	        array(
	            'columnName' => 'CDFILAUXPRAT',
	            'operator' => '=',
	            'value' => $filAux
	        ),
	        array(
	            'columnName' => 'NRPADRAOQUALIDADE',
	            'operator' => '=',
	            'value' => $parameters['NRPADRAOQUALIDADE']
	        )	      
	    );
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    $response->addDataSet(new DataSet("columnsToReturn", $result));
	}

	public function returnReceitaRegiao(DTO\Request\Row $request, DTO\Response $response) {
	    $parameters = $request->getParameters();
	    $plate = $parameters['plate'];
	    $filAux = $parameters['filAux'];
	    $dataSourceName = 'receitaRegiao';
	  
		// condição: CDPRATO, CDFILAUXPRAT, NRPADRAOQUALIDADE, CDREGIAO
  
	    $conditions = array(
	        array(
	            'columnName' => 'CDPRATO',
	            'operator' => '=',
	            'value' => $plate
	        ),
	        array(
	            'columnName' => 'CDFILAUXPRAT',
	            'operator' => '=',
	            'value' => $filAux
	        ),
	        array(
	            'columnName' => 'NRPADRAOQUALIDADE',
	            'operator' => '=',
	            'value' => $parameters['NRPADRAOQUALIDADE']
	        ),
	        array(
	            'columnName' => 'CDREGIAO',
	            'operator' => '=',
	            'value' => $parameters['CDREGIAO']
	        )
	    );
	    $result = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
	    $response->addDataSet(new DataSet("columnsToReturn", $result));
	}
  
	public function saveAjusteQuantidades(DTO\Request\Row $request, DTO\Response $response) {
      $ajusteQtd = $request->getParameters()['ajusteQtd'];

	    $tipoReceita = $ajusteQtd['TIPORECEITA'];
	    $operacao = $ajusteQtd['OPERACAO'];
	    $qnt = $ajusteQtd['QUANTIDADE'];
	    
	    $dadosReceitaDestino = '';
	            
			$newRequest = new DTO\Request\Row('',"","","");
			$newRequest->setParameter('plate', $ajusteQtd['CDPRATO']);
			$newRequest->setParameter('filAux', $ajusteQtd['CDFILAUXPRAT']);
			
	    switch ($tipoReceita) {
	        case 'R':
	            $dadosReceitaDestino = 'receitaRegiao';
	            
        			$newRequest->setParameter('NRPADRAOQUALIDADE', $ajusteQtd['NRPADRAOQUALIDADE']);
        			$newRequest->setParameter('CDREGIAO', $ajusteQtd['CDREGIAO']);
	            
	            $this->returnReceitaRegiao($newRequest, $response);
	            $rows = $response->getDataSets()[0]->getRows();
			        $rows = $this->validRows($rows, $dadosReceitaDestino);
			        if (count($rows) == 0) $response->setError(new DTO\Response\Error('pme::semReceita', 1));
	
	            foreach ($rows as $k => $row) {
                if ($row['IDQTDEFIXREGIAO'] == 'S') {
                  unset($rows[$k]);
                } else {
              		$rows['__is_new'] = false;
  	            	if($operacao == 'M') {
  	            		$row['QTPRODRECREGIAO'] = $row['QTPRODRECREGIAO'] * $qnt;
              			$row['QTBRUTAREGIAOUNR'] = $row['QTBRUTAREGIAOUNR'] * $qnt;
  	            	} else {
  	            	  $row['QTPRODRECREGIAO'] = $row['QTPRODRECREGIAO'] / $qnt;
              			$row['QTBRUTAREGIAOUNR'] = $row['QTBRUTAREGIAOUNR'] / $qnt;
  	            	}
                  $this->insertRecipeRegion($row);
                }
	            }
	        break;
	        case 'Q':
	            $dadosReceitaDestino = 'receitaPadraoQualidade';

        			$newRequest->setParameter('NRPADRAOQUALIDADE', $ajusteQtd['NRPADRAOQUALIDADE']);

	            $this->returnReceitaPadraoQualidade($newRequest, $response);
	            $rows = $response->getDataSets()[0]->getRows();
			        $rows = $this->validRows($rows, $dadosReceitaDestino);
			        if (count($rows) == 0) $response->setError(new DTO\Response\Error('pme::semReceita', 1));

	            foreach ($rows as $k => $row) {
                if ($row['IDQTDEFIXPADQ'] == 'S') {
                  unset($rows[$k]);
                } else {
              		$rows['__is_new'] = false;
  	            	if($operacao == 'M') {
  	            		$row['QTPRODRECPADQ'] = ['QTPRODRECPADQ'] * $qnt;
              			$row['QTBRUTAPADQUNR'] = $row['QTBRUTAPADQUNR'] * $qnt;
  	            	} else {
  	            		$row['QTPRODRECPADQ'] = $row['QTPRODRECPADQ'] / $qnt;
              			$row['QTBRUTAPADQUNR'] = $row['QTBRUTAPADQUNR'] / $qnt;
  	            	}
                  $this->insertRecipeQual($row);
                }
	            }
	        break;
	        case 'P':
	            $dadosReceitaDestino = 'receitaPadrao';
        			
	            $this->returnReceitaPadrao($newRequest, $response);
	            $rows = $response->getDataSets()[0]->getRows();
			        $rows = $this->validRows($rows, $dadosReceitaDestino);
			        if (count($rows) == 0) $response->setError(new DTO\Response\Error('pme::semReceita', 1));
			        
	            foreach ($rows as $k => $row) {
	              if ($row['IDQTDEFIXPR'] == 'S') {
	                unset($rows[$k]);
	              } else {
              		$row['__is_new'] = false;
  	            	if($operacao == 'M') {
  	            		$row['QTPRODPRAT'] = $row['QTPRODPRAT'] * $qnt;
              			$row['QTBRUTAPRUNR'] =$row['QTBRUTAPRUNR'] * $qnt;
          					$row['QTBRUTAPR'] = $row['QTBRUTAPR'] * $qnt;
          					$row['QTCALCNUTRPR'] = $row['QTCALCNUTRPR'] * $qnt;
  	            	} else {
  	            		$row['QTPRODPRAT'] = $row['QTPRODPRAT'] / $qnt;
              			$row['QTBRUTAPRUNR'] = $row['QTBRUTAPRUNR'] / $qnt;
          				  $row['QTBRUTAPR']  = $row['QTBRUTAPR'] / $qnt;
          				  $row['QTCALCNUTRPR']  = $row['QTCALCNUTRPR'] / $qnt;
  	            	}
	            
	                $this->insertRecipeStandard($row);
	              } 
	            }
	        break;
	        case 'S':
	            $dadosReceitaDestino = 'receitaServico';  
	            
        			$newRequest->setParameter('cdfilial', $ajusteQtd['CDFILIAL']);
        			$newRequest->setParameter('cdserv', $ajusteQtd['CDSERVFILI']);
        			
	            $this->returnReceitaServico($newRequest, $response);
	            $rows = $response->getDataSets()[0]->getRows();
			        $rows = $this->validRows($rows, $dadosReceitaDestino);
			        if (count($rows) == 0) $response->setError(new DTO\Response\Error('pme::semReceita', 1));

	            foreach ($rows as $k => $row) {
	              if ($row['IDQTDEFIXSE'] == 'S') {
	                unset($rows[$k]);
	              } else {
              		$rows['__is_new'] = false;
  	            	if($operacao == 'M') {
  	            		$row['QTPRODRECSER'] = $row['QTPRODRECSER'] * $qnt;
              			$row['QTBRUTASEUNR'] = $row['QTBRUTASEUNR'] * $qnt;
          					$row['QTBRUTASE'] = $row['QTBRUTASE'] * $qnt;
      					    $row['QTCALCNUTRSE'] = $row['QTCALCNUTRSE'] * $qnt;
  	            	} else {
              			$row['QTPRODRECSER'] = $row['QTPRODRECSER'] / $qnt;
              			$row['QTBRUTASEUNR'] = $row['QTBRUTASEUNR'] / $qnt;
          			  	$row['QTBRUTASE'] = $row['QTBRUTASE'] / $qnt;
      					  	$row['QTCALCNUTRSE'] = $row['QTCALCNUTRSE'] / $qnt;	
  	            	}
  	            	
  		            $this->insertRecipeService($row);
	              }
	            }
	        break;
	        case 'L':
	            $dadosReceitaDestino = 'receitaLocal';

        			$newRequest->setParameter('cdfilial', $ajusteQtd['CDFILIAL']);

	            $this->returnReceitaLocal($newRequest, $response);
	            $rows = $response->getDataSets()[0]->getRows();
			        $rows = $this->validRows($rows, $dadosReceitaDestino);
			        if (count($rows) == 0) $response->setError(new DTO\Response\Error('pme::semReceita', 1));

	            foreach ($rows as $k => $row) {
	              if ($row['IDQTDEFIXLO'] == 'S') {
	                unset($rows[$k]);
	              } else {
              		$rows['__is_new'] = false;
  	            	if($operacao == 'M') {
  	            		$row['QTPRODRECLOC'] = $row['QTPRODRECLOC'] * $qnt;
            				$row['QTBRUTALOUNR'] = $row['QTBRUTALOUNR'] * $qnt;
          					$row['QTBRUTALO'] = $row['QTBRUTALO'] * $qnt;
      					  	$row['QTCALCNUTRLO'] = $row['QTCALCNUTRLO'] * $qnt;
  	            	} else {
  	            		$row['QTPRODRECLOC'] = $row['QTPRODRECLOC'] / $qnt;
              			$row['QTBRUTALOUNR'] = $row['QTBRUTALOUNR'] / $qnt;
          					$row['QTBRUTALO'] = $row['QTBRUTALO'] / $qnt;
      						  $row['QTCALCNUTRLO'] = $row['QTCALCNUTRLO'] / $qnt;	
  	            	}
  		              $this->insertRecipeLocal($row);
	              }
	            }
	        break;
	        case 'C':
	            $dadosReceitaDestino = 'receitaTpCard';
	            
        			$newRequest->setParameter('tipocard', $ajusteQtd['CDTIPOCARD']);
        			$newRequest->setParameter('central', $ajusteQtd['CDFILCENTPME']);
        			

	            $this->returnReceitaTpCard($newRequest, $response);
	            $rows = $response->getDataSets()[0]->getRows();
			        $rows = $this->validRows($rows, $dadosReceitaDestino);
			        if (count($rows) == 0) $response->setError(new DTO\Response\Error('pme::semReceita', 1));

	            foreach ($rows as $k => $row) {
	              if ($row['IDQTDEFIXPRTP'] == 'S') {
	                unset($rows[$k]);
	              } else {
              		$rows['__is_new'] = false;
  	            	if($operacao == 'M') {
  	            		$row['QTPRODPRATTP'] = $row['QTPRODPRATTP'] * $qnt;
            				$row['QTBRUTAPRUNRTP'] = $row['QTBRUTAPRUNRTP'] * $qnt;
            				$row['QTBRUTAPRTP'] = $row['QTBRUTAPRTP'] * $qnt;
            				$row['QTCALCNUTRPRTP'] = $row['QTCALCNUTRPRTP'] * $qnt;
  	            	} else {
  	            		$row['QTPRODPRATTP'] = $row['QTPRODPRATTP'] / $qnt;
              			$row['QTBRUTAPRUNRTP'] = $row['QTBRUTAPRUNRTP'] / $qnt;
              			$row['QTBRUTAPRTP'] = $row['QTBRUTAPRTP'] / $qnt;
              			$row['QTCALCNUTRPRTP'] = $row['QTCALCNUTRPRTP'] / $qnt;
  	            	}
  	              $this->insertRecipeTpCard($row);
	              }
	            }
	        break;
	    }
	}
}