<?php
namespace PME\Modules\Planejamento\Produto;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class UnidadeAdicionalProdutoController {

	protected $connection;
  protected $environment;
  protected $utilities;

  public function __construct($connection, $environment, $utilities) {
  	$this->connection    = $connection;
    $this->environment   = $environment;
    $this->utilities     = $utilities;
	}

	public function save(DTO\Request $request, DTO\Response $response) {
    try {
      $row = $request->getRow();
      $nrorg = $this->environment->getNrOrg();

      if ($row['OPCRELACAO'] == 0)  $valor = 1/$row['VRFTCONVER'];
      else $valor = $row['VRFTCONVER'];
      
      $query = "INSERT INTO CONVPRODUTO (SGUNIDADE, CDPRODUTO, VRFATOCONVES) VALUES (:SGUNIDADE, :CDPRODUTO, :VRFATOCONVES)";
      $params = array(
        "SGUNIDADE"    => $row["SGUNIDADE"],
        "CDPRODUTO"    => $row["CDPRODUTO"],
        "VRFATOCONVES" => $valor
      );
      
      $stmt = $this->connection->prepare($query);
      $stmt->execute($params);
      $response->addDataSet(new DataSet("save", array(true)));
    } catch (\Exception $e) {
      $message = $e->getMessage();
      $errorMessage = "";
        
      if(stripos($message, "ORA-00001")) $errorMessage = "impossivelAdicionar";
      else $errorMessage = "UNEXPECTED_ERROR";
        
      $response->setError(new Error($errorMessage, 500));
    }
	}
	
	public function deleteUnidade(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $parameters = $request->getParameters();
      $rows = $parameters['checkedRows'];
      $nrorg = $this->utilities->getNrOrg();

      $query = "DELETE FROM CONVPRODUTO WHERE CDPRODUTO = :CDPRODUTO AND NRORG = :NRORG";
      
      foreach($rows as $row) {
        $params = array(
          'CDPRODUTO'  => $row['CDPRODUTO'],
          'NRORG'      => $nrorg
        );
        $this->connection->prepare($query)->execute($params);
      }
      
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
    
  public function deleteConversao(DTO\Request $request, DTO\Response $response) {
    try {
      $parameters = $request->getParameters();
      $rows = $parameters['checkedRows'];

      $this->connection->beginTransaction();

      $query = "DELETE FROM CONVPRODUTO WHERE SGUNIDADE = :SGUNIDADE AND CDPRODUTO = :CDPRODUTO";
      
      foreach($rows as $row){
        $params = array(
          'SGUNIDADE' => $row['SGUNIDADE'],
          'CDPRODUTO' => $row['CDPRODUTO']
        );
        $this->connection->prepare($query)->execute($params);
      }
      $this->connection->commit();
      $response->addDataSet(new DataSet("success", array(true)));
    } catch (\Exception $e) {
      $this->connection->rollback();
      $response->setError(new Error($e->getMessage(), 500));
    }
  }
}