import {
  ModalService, Modal, Form, Select, Grid, IButtonEvent,
} from '@zeedhi/common';
import {
  Loader, Metadata, Config, IEventParam, Http,
} from '@zeedhi/core';
import { PmeMask } from '@/config';

export class Pla01300RevisarQuantidadesController {
  private appController = Loader.getInstance('AppController');

  private utilities = Loader.getInstance('Utilities');

  // private receitaController = Loader.getInstance('ReceitaController');

  // private gridEditController = Loader.getInstance('GridEditableController');
  
  // public prato = Loader.getInstance('Pla01300PratoController');
  
  public modalRevisarQtd!: Modal;
  
  public inicContainer() {
    const project = localStorage.getItem('PRODUCT_ID');
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');

    if (project === '4875') {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      cardapio.isVisible = false;
    }
  }
  
  public openModalRevisarQtd() {
    if (!this.modalRevisarQtd) {
      this.createModalRevisaoQtd().then(() => {
        this.modalRevisarQtd.show();
      });
    } else {
      this.modalRevisarQtd.show();
    }
  }
  
  private async createModalRevisaoQtd() {
    this.modalRevisarQtd = await ModalService.createFromJson('planejamento/prato/modals/pla01303_revisar_qtd', true);
  }

  public closeModalRevisaoQtd() {
    this.modalRevisarQtd.hide();
  }
  
  public fullscreen({ component }: IButtonEvent) {
    this.modalRevisarQtd.fullscreen = !this.modalRevisarQtd.fullscreen;
    component.iconName = (this.modalRevisarQtd.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public changeRecipe() {
    const form: Form = Metadata.getInstance('formFiltroRevisarQtd');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridRevisarQtd');
    const filial = form.getChildInstance<Select>('UNIDADE');
    const cardapio = form.getChildInstance<Select>('TIPOCARD');
    const servico = form.getChildInstance<Select>('SERVICO');
    
    if (row.TPRECEITA === 'L' || row.TPRECEITA1 === 'L') {
      filial.disabled = false;
      servico.disabled = true;
      servico.removeValidation('required');
      servico.setValue('');
      cardapio.disabled = true;
      cardapio.removeValidation('required');
      cardapio.setValue('');
    } else if (row.TPRECEITA === 'C') {
      filial.disabled = true;
      filial.removeValidation('required');
      filial.setValue('');
      servico.disabled = true;
      servico.removeValidation('required');
      servico.setValue('');
      cardapio.disabled = false;
    } else if (row.TPRECEITA === 'S' || row.TPRECEITA1 === 'S') {
      this.changeUnityServ();
      filial.disabled = false;
      filial.setValue('');
      servico.disabled = false;
      cardapio.disabled = true;
      cardapio.removeValidation('required');
      cardapio.setValue('');
    } else if (row.TPRECEITA === 'P' || row.TPRECEITA1 === 'P') {
      // fazer verificacao pro opLogado
      filial.disabled = true;
      servico.disabled = true;
      cardapio.disabled = true;
      filial.removeValidation('required');
      servico.removeValidation('required');
      cardapio.removeValidation('required');
      filial.setValue('');
      servico.setValue('');
      cardapio.setValue('');
    }
    
    grid.datasource.data = [];
    this.setFilterPlate();
  }
  
  public setFilterPlate() {
    const form: Form = Metadata.getInstance('formFiltroRevisarQtd');
    const row = form.value;
    const unidade = Metadata.getInstance<Form>('formUnidade').value.UNIDADEFORM.CDFILIAL;
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
    const grid: Grid = Metadata.getInstance('gridRevisarQtd');
    
    // if (row.TPRECEITA === 'S') this.changeUnityServ();
    
    if (this.utilities.returnPrat(row.PRATOINICIAL, row.PRATOFINAL) && row.PRATOFINAL !== null) {
      this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
      row.PRATOFINAL = null;
    } else if (form.validate()) {
      const conditions = [
        {
          name: 'FILAUX',
          operator: '=',
          value: unidade,
        },
        {
          name: 'PRATOINI',
          operator: '=',
          value: row.PRATOINICIAL,
        },
        {
          name: 'IDRECEITA',
          operator: '=',
          value: row.TPRECEITA !== null ? row.TPRECEITA : row.TPRECEITA1,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: row.SERVICO !== undefined ? row.SERVICO : 'X',
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: row.TIPOCARD !== undefined ? row.TIPOCARD : null,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: this.appController.getCdCentral(),
        },
        {
          name: 'PRATOFIN',
          operator: '=',
          value: row.PRATOFINAL,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: row.UNIDADE !== undefined ? row.UNIDADE : 'T',
        },
        {
          name: 'NIVEL',
          operator: '=',
          value: nivel,
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else grid.datasource.data = [];
  }
  
  public defaultValueFator({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  public defaultValueRecipe({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }
  
  public changeUnityServ() {
    const form: Form = Metadata.getInstance('formFiltroRevisarQtd');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridRevisarQtd');
    
    if (row.TPRECEITA === 'S' || row.TPRECEITA1 === 'S') {
      const serv = Metadata.getInstance<Select>('SERVICO');
      serv.setValue('');
      grid.datasource.data = [];
    }
    this.setFilterPlate();
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formFiltroRevisarQtd');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? 'M' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterGroup({ component }: IEventParam<Select>) {
    const nivel: any = (Config.masks.plateMask as PmeMask).nivelMax;
    component.datasource.filter = {
      CDFILAUXPRAT: 'P',
      CDNVPRATO: nivel,
    };
    component.datasource.get();
  }
  
  public revisarQtd() {
    const form: Form = Metadata.getInstance('formFiltroRevisarQtd');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridRevisarQtd');
    const rowsGrid = grid.selectedRows;
    /* const cdfilcentpme = {
      CDFILCENTPME: this.appController.getCdCentral(),
    }; */
    const cdfilcentpme = this.appController.getCdCentral();
    
    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;
    
    if (form.validate()) {
      if (row.QUANTIDADE === null || row.QUANTIDADE < 0) this.utilities.showAlert('Alert', 'pme::valorAjusteZero', 'error');
      else if (rowsGrid.length > 0) {
        Http.post('/revisarQtd', { row, rowsGrid, cdfilcentpme }).then(() => {
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        }).finally(() => {
          this.closeModalRevisaoQtd();
        });
      } else this.utilities.showAlert('Alert', 'pme::nenhumRegSelecionado', 'error');
    }
  }
}
