import {
  IEventParam, Metadata, Loader, IDictionary, Config,
} from '@zeedhi/core';
import {
  Select, Form, Radio, Button, SelectMultiple,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Pla01303EmissaoReceitasController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
    
  public reportController = Loader.getInstance('ReportController');
  
  public setUnidade = false;

  public project: string = 'PLA';

  public inicContainer() {
    const project = localStorage.getItem('PRODUCT_ID');
    
    /* const form = Metadata.getInstance<Form>('FORM');
    const report = form.value.REPORT.CD_REPORT_CODE; */
    
    const tpcard1 = Metadata.getInstance<Select>('CDTIPOCARD1');
    const tpcard2 = Metadata.getInstance<Select>('CDTIPOCARD2');
    
    if (project === '4875') {
      this.project = 'PAE';
      
      /* if (report === 'pla05500') {
        tpcard1.isVisible = false;
        tpcard2.isVisible = true;
      } else {
        tpcard1.isVisible = true;
        tpcard2.isVisible = false;
      } */
    } else {
      this.project = 'PLA';
      tpcard1.isVisible = false;
      tpcard2.isVisible = false;
    }
  }

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setValues() {
    const form = Metadata.getInstance<Form>('FORM');
    const report = form.value.REPORT.CD_REPORT_CODE;
    const origem = Metadata.getInstance<Radio>('IDTIPORECE');
    origem.isVisible = true;
    origem.disabled = false;

    const grupfili = Metadata.getInstance<Select>('GRUPFILI');
    grupfili.isVisible = false;
    
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const unidade2 = Metadata.getInstance<Select>('CDFILIAL2');
    unidade.isVisible = true;
    unidade2.isVisible = false;
    
    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    servico1.isVisible = true;
    servico2.isVisible = false;
    
    const tpcard1 = Metadata.getInstance<Select>('CDTIPOCARD1');
    const tpcard2 = Metadata.getInstance<Select>('CDTIPOCARD2');
    tpcard1.isVisible = true;
    tpcard2.isVisible = false;
  
    const ordena = Metadata.getInstance<Radio>('ORDENA');
    const ordenaAlf = Metadata.getInstance<Select>('NIVEL');
    ordena.isVisible = false;
    ordenaAlf.isVisible = false;
    
    const pratoini = Metadata.getInstance<Select>('PRATOINI');
    const pratofim = Metadata.getInstance<Select>('PRATOFIM');
    pratoini.isVisible = true;
    pratofim.isVisible = true;
    pratoini.addValidation('required', { message: 'pme::campoObrigatorio' });
    pratofim.addValidation('required', { message: 'pme::campoObrigatorio' });
    
    const prodini = Metadata.getInstance<Select>('PRODINI');
    const prodfim = Metadata.getInstance<Select>('PRODFIN');
    prodini.isVisible = false;
    prodfim.isVisible = false;
    prodini.removeValidation('required');
    prodfim.removeValidation('required');
    
    const tpReq = Metadata.getInstance<Select>('IDTPREQPRODFILI');
    tpReq.isVisible = false;
    tpReq.removeValidation('required');
      
    const nutri = Metadata.getInstance<Select>('CDNUTRIENTE');
    nutri.isVisible = false;
    
    const dtini = Metadata.getInstance<Select>('DTINI');
    const dtfin = Metadata.getInstance<Select>('DTFIN');
    dtfin.isVisible = false;
    dtini.isVisible = false;
    dtini.removeValidation('required');
    dtfin.removeValidation('required');
    
    // inicializa receitas
    grupfili.disabled = true;
    unidade.disabled = true;
    unidade2.disabled = true;
    servico1.disabled = true;
    servico2.disabled = true;
    tpcard1.disabled = true;
    tpcard2.disabled = true;
    
    if (report === 'PLA01300') {
      origem.disabled = true;
      unidade2.removeValidation('required');
      
      ordena.isVisible = true;
      ordenaAlf.isVisible = true;
      ordenaAlf.disabled = true;
    } else if (report === 'PLA01305') {
      origem.disabled = true;
      unidade2.removeValidation('required');
      ordenaAlf.removeValidation('required');
      dtfin.isVisible = true;
      dtini.isVisible = true;
      dtini.addValidation('required', { message: 'pme::campoObrigatorio' });
      dtfin.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else if (report === 'pla01306') {
      origem.disabled = true;
      unidade.isVisible = false;
      unidade2.isVisible = true;
      unidade2.disabled = false;
      unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else if (report === 'pla05500') {
      grupfili.isVisible = true;
      unidade2.removeValidation('required');
      
      prodini.isVisible = true;
      prodfim.isVisible = true;
      prodini.addValidation('required', { message: 'pme::campoObrigatorio' });
      prodfim.addValidation('required', { message: 'pme::campoObrigatorio' });
      
      pratoini.isVisible = false;
      pratofim.isVisible = false;
      pratoini.removeValidation('required');
      pratofim.removeValidation('required');
      this.changeOrigem(report);
    } else if (report === 'pla05000') {
      unidade.isVisible = false;
      unidade2.isVisible = true;
      unidade2.removeValidation('requirefd');

      servico1.isVisible = false;
      servico2.isVisible = true;
      
      tpcard1.isVisible = false;
      tpcard2.isVisible = true;
      
      nutri.isVisible = true;
      
      ordena.isVisible = true;
      ordenaAlf.isVisible = true;
      ordenaAlf.disabled = true;
      
      this.changeOrigem(report);
    } else if (report === 'pla05400') {
      origem.isVisible = true;
      origem.disabled = true;
      unidade2.removeValidation('required');
      servico2.removeValidation('required');
      
      grupfili.isVisible = true;
      grupfili.disabled = false;
      unidade.disabled = false;
      
      prodini.isVisible = true;
      prodfim.isVisible = true;
      prodini.addValidation('required', { message: 'pme::campoObrigatorio' });
      prodfim.addValidation('required', { message: 'pme::campoObrigatorio' });
      ordenaAlf.removeValidation('required');
      tpReq.isVisible = true;
      tpReq.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else if (report === 'pla013B') {
      unidade.isVisible = false;
      unidade2.isVisible = true;
      unidade2.removeValidation('required');

      servico1.isVisible = false;
      servico2.isVisible = true;
      
      tpcard1.isVisible = false;
      tpcard2.isVisible = true;
      ordenaAlf.removeValidation('required');
      nutri.isVisible = true;
      
      this.changeOrigem(report);
    }
    
    this.clearFields();
  }
  
  public changeOrigem(report: string) {
    const origem = Metadata.getInstance<Radio>('IDTIPORECE');
    
    const grupfili = Metadata.getInstance<Select>('GRUPFILI');
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const unidade2 = Metadata.getInstance<Select>('CDFILIAL2');
    unidade2.removeValidation('required');
    
    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    servico2.removeValidation('required');
    
    const tpcard1 = Metadata.getInstance<Select>('CDTIPOCARD1');
    const tpcard2 = Metadata.getInstance<Select>('CDTIPOCARD2');
    
    if (origem.value === 'P') {
      grupfili.disabled = true;
      unidade.disabled = true;
      unidade2.disabled = true;
      servico1.disabled = true;
      servico2.disabled = true;
      
      unidade2.removeValidation('required');
      servico2.removeValidation('required');
      
      tpcard1.disabled = true;
      tpcard2.disabled = true;
    } else if (origem.value === 'F') {
      grupfili.disabled = false;
      unidade.disabled = false;
      unidade2.disabled = false;
      servico1.disabled = true;
      servico2.disabled = true;
      
      if (report === 'pla05000' || report === 'pla013B') {
        unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
        servico2.removeValidation('required');
      } else {
        unidade2.removeValidation('required');
        servico2.removeValidation('required');
      }
      
      tpcard1.disabled = true;
      tpcard2.disabled = true;
    } else if (origem.value === 'S') {
      grupfili.disabled = false;
      unidade.disabled = false;
      unidade2.disabled = false;
      servico1.disabled = false;
      servico2.disabled = false;
      
      if (report === 'pla05000' || report === 'pla013B') {
        unidade2.addValidation('required', { message: 'pme::campoObrigatorio' });
        servico2.addValidation('required', { message: 'pme::campoObrigatorio' });
      } else {
        unidade2.removeValidation('required');
        servico2.removeValidation('required');
      }
      
      tpcard1.disabled = true;
      tpcard2.disabled = true;
    } else if (origem.value === 'T') {
      grupfili.disabled = true;
      unidade.disabled = true;
      unidade2.disabled = true;
      servico1.disabled = true;
      servico2.disabled = true;
      
      unidade2.removeValidation('required');
      servico2.removeValidation('required');
      
      tpcard1.disabled = false;
      tpcard2.disabled = false;
    }
  }
  
  public clearUnidade() {
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const unidade2 = Metadata.getInstance<Select>('CDFILIAL2');
    
    unidade.setValue('');
    unidade2.setValue('');
    
    this.clearServico();
  }
  
  public clearServico() {
    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    servico1.setValue('');
    servico2.setValue('');
  }
  
  public clearFields() {
    const unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
    const servico1 = Metadata.getInstance<Select>('CDSERVFILI1');
    const servico2 = Metadata.getInstance<Select>('CDSERVFILI2');
    const tpCard1 = Metadata.getInstance<Select>('CDTIPOCARD1');
    const tpCard2 = Metadata.getInstance<Select>('CDTIPOCARD2');

    const pratoini = Metadata.getInstance<Select>('PRATOINI');
    const pratofim = Metadata.getInstance<Select>('PRATOFIM');
    
    this.utilities.clearPeriodoParams();

    servico1.setValue('');
    servico2.setValue('');
    unidade.setValue('');
    tpCard1.setValue('');
    tpCard2.setValue('');
    pratoini.setValue('');
    pratofim.setValue('');
  }
  
  public setFilterTpCardapio() {
    const cardapio1 = Metadata.getInstance<SelectMultiple>('CDTIPOCARD1');
    const cardapio2 = Metadata.getInstance<SelectMultiple>('CDTIPOCARD2');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    const form = Metadata.getInstance<Form>('FORM');
    const report = form.value.REPORT.CD_REPORT_CODE;
    
    if (report === 'pla05500') {
      cardapio1.datasource.filter = filter;
      cardapio1.datasource.get();
    } else {
      cardapio2.datasource.filter = filter;
      cardapio2.datasource.get();
    }
  }
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const filter = [
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServAgrupUnid({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('FORM');
    const report = form.value.REPORT.CD_REPORT_CODE;
    let unidade;
    let unidadeValue;
    
    if (report === 'pla05000' || report === 'pla013B') {
      unidade = Metadata.getInstance<Select>('CDFILIAL2');
      unidadeValue = (unidade.value) ? unidade.value : 'T';
    } else {
      unidade = Metadata.getInstance<SelectMultiple>('CDFILIAL1');
      unidadeValue = (unidade.value) ? unidade.value.join('_') : 'T';
    }
    
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const agrupUnidValue = agrupUnid.value ? agrupUnid.value : 'T';
    
    const grupserv = Metadata.getInstance<Select>('GRUPSERV');
    const grupServValue = grupserv.value ? grupserv.value : 'T';
    
    const filter = [
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnidValue,
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidadeValue,
      },
      {
        name: 'CDGRUPSERV',
        operator: '=',
        value: grupServValue,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.project === 'PAE' ? 'M' : 'N',
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public controllPratoIni() {
    const pratoFinal = Metadata.getInstance<Select>('PRATOFIM');
    if (pratoFinal.value !== null) pratoFinal.setValue(null);
  }
  
  public verifyPratIni({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('FORM');
    if (component.value !== null) {
      if (this.utilities.returnPrat(form.getChildInstance<Select>('PRATOINI').value, component.value)) {
        this.utilities.showAlert('Alert', 'pme::pratoIniMaior', 'error');
        component.setValue(null);
      }
    }
  }
  
  public verifyProductIni({ component }: IEventParam<Select>) {
    if (!(Metadata.getInstance<Select>('PRODINI'))) {
      const productIni = Metadata.getInstance<Select>('PRODINI').value.CDARVPROD;
      const productFin = Metadata.getInstance<Select>('PRODFIN').value.CDARVPROD;
      
      if (productIni > productFin) {
        this.utilities.showAlert('Alert', 'pme::prodIniMaior', 'error');
        component.setValue(null);
      }
    }
  }
  
  public controllProductIni() {
    const productFin = Metadata.getInstance<Select>('PRODFIN');
    if (productFin.value !== null) productFin.setValue(null);
  }
  
  public setOrdena() {
    const ordena = Metadata.getInstance<Radio>('ORDENA');
    const ordenaAlf = Metadata.getInstance<Select>('NIVEL');
      
    if (ordena.value === '1') {
      ordenaAlf.disabled = false;
      ordenaAlf.addValidation('required', { message: 'pme::campoObrigatorio' });
    } else {
      ordenaAlf.disabled = true;
      ordenaAlf.removeValidation('required');
    }
  }

  public validaPDF({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORM');
    if (this.utilities.centralIsValid() && form.validate()) {
      const row = form.value;
      this.setParams(row);
      if (!row.CDFILCENTPME && this.project === 'PAE') this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.reportFormat = component.name;
        this.utilities.activeLoadingButton(component);
        this.reportController.generateReport('report/emissaoReceitas', row, row.CD_REPORT_CODE).finally(() => {
          this.utilities.disableLoadingButton(component);
        });
      }
    }
  }
  
  private setParams(row: IDictionary) {
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    
    const maskProd: string = (Config.masks.productMask as PmeMask).pattern;
    const maskPrat: string = (Config.masks.plateMask as PmeMask).pattern;
    row.CDPICTPROD = this.utilities.formatReportMask(maskProd);
    row.CDPICTPRAT = this.utilities.formatReportMask(maskPrat);

    row.CDFILCENTPME = this.appController.getCdCentral();
    
    const form = Metadata.getInstance<Form>('FORM');
    const report = form.value.REPORT.CD_REPORT_CODE;
    
    row.IDTIPORECE = row.IDTIPORECE ? row.IDTIPORECE : 'P';
    row.GRUPFILI = row.GRUPFILI ? row.GRUPFILI : null;
    
    row.TIPOCARD = row.CDTIPOCARD1 ? row.CDTIPOCARD1 : null;
    row.CDFILIALAUX = this.utilities.getCdCentralLogada;
    
    if (report === 'pla01306') {
      row.CDFILIAL = row.CDFILIAL2;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    } else if (report === 'PLA01300') {
      row.CDFILIAL = this.utilities.getCdCentralLogada;
      row.CDFILCENTPME = this.utilities.getCdCentralLogada;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    } else if (report === 'pla05000') {
      row.CDFILIAL = this.utilities.getCdCentralLogada;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI2 : 'X';
    } else if (report === 'pla05500') {
      row.CDFILIAL = row.CDFILIAL1;
      row.TIPOCARD = row.CDTIPOCARD2 ? row.CDTIPOCARD2 : null;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    } else if (report === 'pla013B') {
      row.CDFILIAL = row.CDFILIAL2 ? row.CDFILIAL2 : this.utilities.getCdCentralLogada;
      row.CDSERVFILI = row.CDSERVFILI2 ? row.CDSERVFILI2 : 'T';
    } else {
      row.CDFILIAL = row.CDFILIAL1;
      row.CDSERVFILI = row.CDSERVFILI1 ? row.CDSERVFILI1 : [];
    }
    
    row.PRATOINI = row.PRATOINI ? row.PRATOINI : '0';
    row.PRATOFIM = row.PRATOFIM ? row.PRATOFIM : '0';
    row.MAXNIVEL = (Config.masks.plateMask as PmeMask).nivelMax;
    
    row.PRODINI = row.PRODINI ? row.PRODINI : '0';
    row.PRODFIN = row.PRODFIN ? row.PRODFIN : '0';
    
    row.CDFILIPROC = this.utilities.getCdCentralLogada;
    
    row.GRUPSERV = row.GRUPSERV ? row.GRUPSERV : null;
    row.IDTIPOSERV = this.project === 'PAE' ? 'M' : 'N';
    
    row.ORDENA = row.ORDENA ? row.ORDENA : '0';
    
    row.DTINI = row.DTINI ? row.DTINI : null;
    row.DTFIN = row.DTFIN ? row.DTFIN : null;
    
    row.CDNUTRIENTE = row.CDNUTRIENTE ? row.CDNUTRIENTE : null;
    row.PROJECT = this.project;
    row.IDTPREQPRODFILI = row.IDTPREQPRODFILI ? row.IDTPREQPRODFILI : null;
  }
}
